@@ -398,20 +398,14 @@ class ChatRooms(IconNotebook):
         for room in self.joinedrooms.values():
             room.get_user_status(msg.user, msg.status)
 
-    def set_user_flag(self, user, flag):
+    def set_user_flag(self, user, country):
         for room in self.joinedrooms.values():
-            room.set_user_flag(user, flag)
-
-    def get_user_address(self, user):
-
-        if user not in self.frame.np.users or self.frame.np.users[user].addr is None:
-            self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
+            room.set_user_flag(user, country)
 
     def user_joined_room(self, msg):
 
         if msg.room in self.joinedrooms:
             self.joinedrooms[msg.room].user_joined_room(msg.username, msg.userdata)
-            self.get_user_address(msg.username)
 
     def user_left_room(self, msg):
         self.joinedrooms[msg.room].user_left_room(msg.username)
@@ -469,7 +463,6 @@ class ChatRooms(IconNotebook):
         room = self.joinedrooms[msg.room]
 
         self.remove_page(room.Main)
-        room.destroy()
         del self.joinedrooms[msg.room]
 
         if msg.room[-1:] != ' ':  # meta rooms
@@ -576,14 +569,15 @@ class ChatRoom:
         if room not in config["columns"]["chat_room"]:
             config["columns"]["chat_room"][room] = {}
 
+        self.column_numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8]
         self.cols = cols = initialise_columns(
             ("chat_room", room),
             self.UserList,
-            ["status", _("Status"), 25, "pixbuf", None, None],
-            ["country", _("Country"), 25, "pixbuf", None, None],
-            ["user", _("User"), 100, "text", self.user_column_draw, None],
-            ["speed", _("Speed"), 100, "number", None, None],
-            ["files", _("Files"), 100, "number", None, None]
+            ["status", _("Status"), 25, "pixbuf", None],
+            ["country", _("Country"), 25, "pixbuf", None],
+            ["user", _("User"), 100, "text", self.user_column_draw],
+            ["speed", _("Speed"), 100, "number", None],
+            ["files", _("Files"), 100, "number", None]
         )
 
         cols["status"].set_sort_column_id(5)
@@ -602,35 +596,19 @@ class ChatRoom:
         self.users = {}
 
         self.usersmodel = Gtk.ListStore(
-            GObject.TYPE_OBJECT,
-            GObject.TYPE_OBJECT,
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GObject.TYPE_INT,
-            GObject.TYPE_INT,
-            GObject.TYPE_INT,
-            GObject.TYPE_STRING
+            GObject.TYPE_OBJECT,  # (0)  status_image
+            GObject.TYPE_OBJECT,  # (1)  flag
+            str,                  # (2)  username
+            str,                  # (3)  h_speed
+            str,                  # (4)  h_files
+            int,                  # (5)  status
+            GObject.TYPE_UINT64,  # (6)  avgspeed
+            GObject.TYPE_UINT64,  # (7)  files
+            str                   # (8)  country
         )
 
-        for username, user in users.items():
-
-            img = self.frame.get_status_image(user.status)
-            flag = user.country
-
-            if flag:
-                flag = "flag_" + flag
-                self.frame.flag_users[username] = flag
-            else:
-                flag = self.frame.get_user_flag(username)
-
-            hspeed = human_speed(user.avgspeed)
-            hfiles = humanize(user.files)
-            iterator = self.usersmodel.append(
-                [img, self.frame.get_flag_image(flag), username, hspeed, hfiles, user.status, user.avgspeed, user.files, flag]
-            )
-            self.users[username] = iterator
-            self.chatrooms.get_user_address(username)
+        for username, userdata in users.items():
+            self.add_user_row(username, userdata)
 
         self.usersmodel.set_sort_column_id(2, Gtk.SortType.ASCENDING)
 
@@ -684,6 +662,33 @@ class ChatRoom:
 
         self.count_users()
 
+    def add_user_row(self, username, userdata):
+
+        status_image = self.frame.get_status_image(userdata.status)
+
+        # Request user's IP address, so we can get the country
+        self.frame.np.queue.put(slskmessages.GetPeerAddress(username))
+
+        hspeed = human_speed(userdata.avgspeed)
+        hfiles = humanize(userdata.files)
+
+        iterator = self.usersmodel.insert_with_valuesv(
+            -1, self.column_numbers,
+            [
+                GObject.Value(GObject.TYPE_OBJECT, status_image),
+                GObject.Value(GObject.TYPE_OBJECT, None),
+                username,
+                hspeed,
+                hfiles,
+                userdata.status,
+                GObject.Value(GObject.TYPE_UINT64, userdata.avgspeed),
+                GObject.Value(GObject.TYPE_UINT64, userdata.files),
+                ""
+            ]
+        )
+
+        self.users[username] = iterator
+
     def room_status(self, column, cellrenderer, model, iterator, dummy='dummy'):
         # cellrenderer.set_property("weight", colour)
         pass
@@ -751,9 +756,9 @@ class ChatRoom:
                 line = re.sub(r"\\s\\s+", "  ", line)
 
                 if user != config["server"]["login"]:
-                    append_line(self.ChatScroll, self.frame.censor_chat(line), tag, username=user, usertag=usertag, timestamp_format="")
+                    append_line(self.ChatScroll, self.frame.censor_chat(line), tag, username=user, usertag=usertag, timestamp_format="", scroll=False)
                 else:
-                    append_line(self.ChatScroll, line, tag, username=user, usertag=usertag, timestamp_format="")
+                    append_line(self.ChatScroll, line, tag, username=user, usertag=usertag, timestamp_format="", scroll=False)
 
             if len(lines) > 0:
                 append_line(self.ChatScroll, _("--- old messages above ---"), self.tag_hilite)
@@ -764,9 +769,6 @@ class ChatRoom:
     def on_find_room_log(self, widget):
         self.ChatSearchBar.set_search_mode(True)
 
-    def destroy(self):
-        self.Main.destroy()
-
     def get_selected_username(self, treeview):
 
         model, iterator = treeview.get_selection().get_selected()
@@ -986,7 +988,7 @@ class ChatRoom:
             widget.set_text("")
             return
 
-        s = text.split(" ", 1)  # string
+        s = text.split(" ", 1)
         cmd = s[0]
 
         if len(s) == 2:
@@ -994,116 +996,122 @@ class ChatRoom:
         else:
             args = ""
 
-        byteargs = args.encode('utf-8')  # bytes
-
         if cmd in ("/alias", "/al"):
             append_line(self.ChatScroll, add_alias(args), self.tag_remote, "")
+
             if self.frame.np.config.sections["words"]["aliases"]:
                 self.frame.chatrooms.update_completions()
                 self.frame.privatechats.update_completions()
 
         elif cmd in ("/unalias", "/un"):
             append_line(self.ChatScroll, unalias(args), self.tag_remote, "")
+
             if self.frame.np.config.sections["words"]["aliases"]:
                 self.frame.chatrooms.update_completions()
                 self.frame.privatechats.update_completions()
 
-        elif cmd in ["/w", "/whois", "/info"]:
-            if byteargs:
-                self.frame.local_user_info_request(byteargs)
-                self.frame.on_user_info(None)
+        elif cmd in ("/w", "/whois", "/info"):
+            if args:
+                self.frame.local_user_info_request(args)
+                self.frame.change_main_page("userinfo")
 
-        elif cmd in ["/b", "/browse"]:
-            if byteargs:
-                self.frame.browse_user(byteargs)
-                self.frame.on_user_browse(None)
+        elif cmd in ("/b", "/browse"):
+            if args:
+                self.frame.browse_user(args)
+                self.frame.change_main_page("userbrowse")
 
         elif cmd == "/ip":
-            if byteargs:
-                user = byteargs
+            if args:
+                user = args
                 self.frame.np.ip_requested.add(user)
                 self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
 
         elif cmd == "/pm":
-            if byteargs:
-                self.frame.privatechats.send_message(byteargs, show_user=True)
+            if args:
+                self.frame.privatechats.send_message(args, show_user=True)
                 self.frame.change_main_page("private")
 
-        elif cmd in ["/m", "/msg"]:
-            if byteargs:
-                user = byteargs.split(b" ", 1)[0]
-                try:
-                    msg = args.split(" ", 1)[1]
-                except IndexError:
+        elif cmd in ("/m", "/msg"):
+            if args:
+                s = args.split(" ", 1)
+                user = s[0]
+                if len(s) == 2:
+                    msg = s[1]
+                else:
                     msg = None
-                self.frame.privatechats.send_message(user, msg)
+                self.frame.privatechats.send_message(user, msg, show_user=True)
+                self.frame.change_main_page("private")
 
-        elif cmd in ["/s", "/search"]:
+        elif cmd in ("/s", "/search"):
             if args:
                 self.frame.searches.do_search(args, 0)
                 self.frame.on_search(None)
+                self.frame.change_main_page("search")
 
-        elif cmd in ["/us", "/usearch"]:
-            s = byteargs.split(" ", 1)
+        elif cmd in ("/us", "/usearch"):
+            s = args.split(" ", 1)
             if len(s) == 2:
                 self.frame.searches.do_search(s[1], 3, [s[0]])
                 self.frame.on_search(None)
+                self.frame.change_main_page("search")
 
-        elif cmd in ["/rs", "/rsearch"]:
+        elif cmd in ("/rs", "/rsearch"):
             if args:
                 self.frame.searches.do_search(args, 1)
                 self.frame.on_search(None)
+                self.frame.change_main_page("search")
 
-        elif cmd in ["/bs", "/bsearch"]:
+        elif cmd in ("/bs", "/bsearch"):
             if args:
                 self.frame.searches.do_search(args, 2)
                 self.frame.on_search(None)
+                self.frame.change_main_page("search")
 
-        elif cmd in ["/j", "/join"]:
-            if byteargs:
-                self.frame.np.queue.put(slskmessages.JoinRoom(byteargs))
+        elif cmd in ("/j", "/join"):
+            if args:
+                self.frame.np.queue.put(slskmessages.JoinRoom(args))
 
-        elif cmd in ["/l", "/leave", "/p", "/part"]:
-            if byteargs:
-                self.frame.np.queue.put(slskmessages.LeaveRoom(byteargs))
+        elif cmd in ("/l", "/leave", "/p", "/part"):
+            if args:
+                self.frame.np.queue.put(slskmessages.LeaveRoom(args))
             else:
                 self.frame.np.queue.put(slskmessages.LeaveRoom(self.room))
 
-        elif cmd in ["/ad", "/add", "/buddy"]:
-            if byteargs:
-                self.frame.userlist.add_to_list(byteargs)
+        elif cmd in ("/ad", "/add", "/buddy"):
+            if args:
+                self.frame.userlist.add_to_list(args)
 
-        elif cmd in ["/rem", "/unbuddy"]:
-            if byteargs:
-                self.frame.userlist.remove_from_list(byteargs)
+        elif cmd in ("/rem", "/unbuddy"):
+            if args:
+                self.frame.userlist.remove_from_list(args)
 
         elif cmd == "/ban":
-            if byteargs:
-                self.frame.ban_user(byteargs)
+            if args:
+                self.frame.ban_user(args)
 
         elif cmd == "/ignore":
-            if byteargs:
-                self.frame.ignore_user(byteargs)
+            if args:
+                self.frame.ignore_user(args)
 
         elif cmd == "/ignoreip":
-            if byteargs:
-                self.frame.ignore_ip(byteargs)
+            if args:
+                self.frame.ignore_ip(args)
 
         elif cmd == "/unban":
-            if byteargs:
-                self.frame.unban_user(byteargs)
+            if args:
+                self.frame.unban_user(args)
 
         elif cmd == "/unignore":
-            if byteargs:
-                self.frame.unignore_user(byteargs)
+            if args:
+                self.frame.unignore_user(args)
 
-        elif cmd in ["/clear", "/cl"]:
+        elif cmd in ("/clear", "/cl"):
             self.ChatScroll.get_buffer().set_text("")
 
-        elif cmd in ["/a", "/away"]:
+        elif cmd in ("/a", "/away"):
             self.frame.on_away(None)
 
-        elif cmd in ["/q", "/quit", "/exit"]:
+        elif cmd in ("/q", "/quit", "/exit"):
             self.frame.on_quit(None)
             return  # Avoid gsignal warning
 
@@ -1111,7 +1119,6 @@ class ChatRoom:
             self.display_now_playing()
 
         elif cmd == "/rescan":
-
             # Rescan public shares if needed
             if not self.frame.np.config.sections["transfers"]["friendsonly"] and self.np.config.sections["transfers"]["shared"]:
                 self.frame.on_rescan()
@@ -1120,25 +1127,24 @@ class ChatRoom:
             if self.frame.np.config.sections["transfers"]["enablebuddyshares"]:
                 self.frame.on_buddy_rescan()
 
-        elif cmd in ["/tick", "/t"]:
+        elif cmd in ("/tick", "/t"):
             self.frame.np.queue.put(slskmessages.RoomTickerSet(self.room, args))
 
-        elif cmd in ("/tickers",):
+        elif cmd == "/tickers":
             self.show_tickers()
 
-        elif cmd in ('/toggle',):
-            if byteargs:
-                self.frame.np.pluginhandler.toggle_plugin(byteargs)
+        elif cmd == "/toggle":
+            if args:
+                self.frame.np.pluginhandler.toggle_plugin(args)
 
         elif cmd[:1] == "/" and self.frame.np.pluginhandler.trigger_public_command_event(self.room, cmd[1:], args):
             pass
 
-        elif cmd and cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
+        elif cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
             log.add(_("Command %s is not recognized"), text)
             return
 
         else:
-
             if text[:2] == "//":
                 text = text[1:]
 
@@ -1177,21 +1183,8 @@ class ChatRoom:
         if username not in self.frame.np.config.sections["server"]["ignorelist"] and not self.frame.user_ip_is_ignored(username):
             append_line(self.RoomLog, _("%s joined the room") % username, self.tag_log)
 
-        img = self.frame.get_status_image(userdata.status)
-        flag = userdata.country
-
-        if flag is not None:
-            flag = "flag_" + flag
-            self.frame.flag_users[username] = flag
-        else:
-            flag = self.frame.get_user_flag(username)
-
-        hspeed = human_speed(userdata.avgspeed)
-        hfiles = humanize(userdata.files)
-
-        self.users[username] = self.usersmodel.append(
-            [img, self.frame.get_flag_image(flag), username, hspeed, hfiles, userdata.status, userdata.avgspeed, userdata.files, flag]
-        )
+        self.frame.np.pluginhandler.user_join_chatroom_notification(self.room, username)
+        self.add_user_row(username, userdata)
 
         self.get_user_tag(username)
         self.count_users()
@@ -1222,6 +1215,8 @@ class ChatRoom:
         if username not in self.frame.np.config.sections["server"]["ignorelist"] and not self.frame.user_ip_is_ignored(username):
             append_line(self.RoomLog, _("%s left the room") % username, self.tag_log)
 
+        self.frame.np.pluginhandler.user_leave_chatroom_notification(self.room, username)
+
         self.usersmodel.remove(self.users[username])
         del self.users[username]
 
@@ -1256,7 +1251,10 @@ class ChatRoom:
         if user not in self.users:
             return
 
-        self.usersmodel.set(self.users[user], 3, human_speed(avgspeed), 4, humanize(files), 6, avgspeed, 7, files)
+        self.usersmodel.set_value(self.users[user], 3, human_speed(avgspeed))
+        self.usersmodel.set_value(self.users[user], 4, humanize(files))
+        self.usersmodel.set_value(self.users[user], 6, GObject.Value(GObject.TYPE_UINT64, avgspeed))
+        self.usersmodel.set_value(self.users[user], 7, GObject.Value(GObject.TYPE_UINT64, files))
 
     def get_user_status(self, user, status):
 
@@ -1279,14 +1277,16 @@ class ChatRoom:
             color = self.get_user_status_color(status)
             self.update_tag_visuals(self.tag_users[user], color)
 
-        self.usersmodel.set(self.users[user], 0, img, 5, status)
+        self.usersmodel.set_value(self.users[user], 0, GObject.Value(GObject.TYPE_OBJECT, img))
+        self.usersmodel.set_value(self.users[user], 5, status)
 
-    def set_user_flag(self, user, flag):
+    def set_user_flag(self, user, country):
 
         if user not in self.users:
             return
 
-        self.usersmodel.set(self.users[user], 1, self.frame.get_flag_image(flag), 8, flag)
+        self.usersmodel.set_value(self.users[user], 1, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)))
+        self.usersmodel.set_value(self.users[user], 8, country)
 
     def create_tag(self, buffer, color, username=None):
 
@@ -1460,27 +1460,12 @@ class ChatRoom:
         self.usersmodel.set_default_sort_func(lambda *args: -1)
         self.usersmodel.set_sort_column_id(-1, Gtk.SortType.ASCENDING)
 
-        for (username, user) in users.items():
+        for username, userdata in users.items():
 
             if username in self.users:
                 self.usersmodel.remove(self.users[username])
 
-            img = self.frame.get_status_image(user.status)
-            flag = user.country
-
-            if flag is not None:
-                flag = "flag_" + flag
-                self.frame.flag_users[username] = flag
-            else:
-                flag = self.frame.get_user_flag(username)
-
-            hspeed = human_speed(user.avgspeed)
-            hfiles = humanize(user.files)
-
-            myiter = self.usersmodel.append([img, self.frame.get_flag_image(flag), username, hspeed, hfiles, user.status, user.avgspeed, user.files, flag])
-
-            self.users[username] = myiter
-            self.chatrooms.get_user_address(username)
+            self.add_user_row(username, userdata)
 
         self.UserList.set_sensitive(True)
 
@@ -1635,7 +1620,7 @@ class ChatRoom:
         return True
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
-        return show_country_tooltip(widget, x, y, tooltip, 8)
+        return show_country_tooltip(widget, x, y, tooltip, 8, stripprefix='')
 
     def on_log_toggled(self, widget):
 
