@@ -94,7 +94,6 @@ class NicotineFrame:
         self.rescanning = False
         self.brescanning = False
         self.needrescan = False
-        self.away = False
         self.autoaway = False
         self.awaytimerid = None
         self.shutdown = False
@@ -125,6 +124,10 @@ class NicotineFrame:
 
         config = self.np.config.sections
 
+        """ Previous away state """
+
+        self.away = config["server"]["away"]
+
         """ GTK Settings """
 
         dark_mode = config["ui"]["dark_mode"]
@@ -151,6 +154,9 @@ class NicotineFrame:
         self.set_up_actions()
         self.set_up_menu()
 
+        self.accel_group = Gtk.AccelGroup()
+        self.MainWindow.add_accel_group(self.accel_group)
+
         """ Icons """
 
         self.load_icons()
@@ -432,10 +438,6 @@ class NicotineFrame:
         self.uploads.init_interface(self.np.transfers.uploads)
         self.downloads.init_interface(self.np.transfers.downloads)
 
-        for i in self.np.config.sections["server"]["userlist"]:
-            user = i[0]
-            self.np.queue.put(slskmessages.AddUser(user))
-
         if msg.banner != "":
             log.add(msg.banner)
 
@@ -450,23 +452,23 @@ class NicotineFrame:
         except (ImportError, ValueError):
             self.spell_checker = False
 
-    def get_flag_image(self, flag):
+    def get_flag_image(self, country):
 
-        if flag is None:
-            return
+        if not country:
+            return None
 
-        flag = flag.lower()
+        country = country.lower().replace("flag_", "")
 
         try:
-            if flag not in self.flag_images:
-                self.flag_images[flag] = GdkPixbuf.Pixbuf.new_from_file(
-                    os.path.join(self.gui_dir, "icons", "flags", flag[5:] + ".svg")
+            if country not in self.flag_images:
+                self.flag_images[country] = GdkPixbuf.Pixbuf.new_from_file(
+                    os.path.join(self.gui_dir, "icons", "flags", country + ".svg")
                 )
 
         except Exception:
             return None
 
-        return self.flag_images[flag]
+        return self.flag_images[country]
 
     def load_ui_icon(self, name):
         """ Load icon required by the UI """
@@ -572,7 +574,6 @@ class NicotineFrame:
 
         self.images = {}
         self.flag_images = {}
-        self.flag_users = {}
 
         names = [
             "away",
@@ -644,6 +645,15 @@ class NicotineFrame:
         if self.autoaway:
             self.autoaway = self.away = False
 
+        self.uploads.conn_close()
+        self.downloads.conn_close()
+        self.searches.wish_list.conn_close()
+        self.userlist.conn_close()
+
+        if self.shutdown:
+            # Application is shutting down, stop here
+            return
+
         self.set_widget_online_status(False)
         self.tray.set_connected(False)
 
@@ -652,10 +662,6 @@ class NicotineFrame:
         self.searches.wish_list.interval = 0
         self.chatrooms.conn_close()
         self.privatechats.conn_close()
-        self.searches.wish_list.conn_close()
-        self.uploads.conn_close()
-        self.downloads.conn_close()
-        self.userlist.conn_close()
         self.userinfo.conn_close()
         self.userbrowse.conn_close()
 
@@ -725,7 +731,8 @@ class NicotineFrame:
         self.disconnect_action.connect("activate", self.on_disconnect)
         self.application.add_action(self.disconnect_action)
 
-        self.away_action = Gio.SimpleAction.new_stateful("away", None, GLib.Variant.new_boolean(False))
+        state = self.np.config.sections["server"]["away"]
+        self.away_action = Gio.SimpleAction.new_stateful("away", None, GLib.Variant.new_boolean(state))
         self.away_action.connect("change-state", self.on_away)
         self.application.add_action(self.away_action)
 
@@ -908,9 +915,11 @@ class NicotineFrame:
         self.np.queue.put(slskmessages.ConnClose(self.np.active_server_conn))
 
     def on_away(self, *args):
-
         self.away = not self.away
+        self.np.config.sections["server"]["away"] = self.away
+        self._apply_away_state()
 
+    def _apply_away_state(self):
         if not self.away:
             self.set_user_status(_("Online"))
             self.on_disable_auto_away()
@@ -921,7 +930,6 @@ class NicotineFrame:
 
         self.np.queue.put(slskmessages.SetStatus(self.away and 1 or 2))
         self.away_action.set_state(GLib.Variant.new_boolean(self.away))
-
         self.privatechats.update_visuals()
 
     def on_check_privileges(self, *args):
@@ -1259,11 +1267,6 @@ class NicotineFrame:
 
         load_ui_elements(self, os.path.join(self.gui_dir, "ui", "dialogs", "about.ui"))
 
-        # Remove non-functional close button added by GTK
-        buttons = self.AboutDialog.get_action_area().get_children()
-        if buttons:
-            buttons[-1].destroy()
-
         # Override link handler with our own
         self.AboutDialog.connect("activate-link", self.on_about_uri)
 
@@ -1275,7 +1278,8 @@ class NicotineFrame:
         self.AboutDialog.set_transient_for(self.MainWindow)
         self.AboutDialog.set_version(version)
 
-        self.AboutDialog.show()
+        self.AboutDialog.run()
+        self.AboutDialog.destroy()
 
     def on_about_uri(self, widget, uri):
         open_uri(uri, self.MainWindow)
@@ -1298,6 +1302,9 @@ class NicotineFrame:
         end_widget = self.__dict__[page_id + "End"]
         end_widget.add(self.HeaderMenu)
 
+        key, mod = Gtk.accelerator_parse("F10")
+        self.HeaderMenu.add_accelerator("clicked", self.accel_group, key, mod, Gtk.AccelFlags.VISIBLE)
+
         header_bar = self.__dict__["Header" + page_id]
         header_bar.set_title(GLib.get_application_name())
         self.MainWindow.set_titlebar(header_bar)
@@ -1319,12 +1326,9 @@ class NicotineFrame:
         toolbar_contents = self.__dict__[page_id + "ToolbarContents"]
 
         title_widget = self.__dict__[page_id + "Title"]
+        title_widget.set_hexpand(True)
         header_bar.set_custom_title(None)
-        toolbar_contents.pack_start(title_widget, True, True, 0)
-
-        end_widget = self.__dict__[page_id + "End"]
-        header_bar.remove(end_widget)
-        toolbar_contents.pack_end(end_widget, False, False, 0)
+        toolbar_contents.add(title_widget)
 
         try:
             start_widget = self.__dict__[page_id + "Start"]
@@ -1335,12 +1339,19 @@ class NicotineFrame:
             # No start widget
             pass
 
+        end_widget = self.__dict__[page_id + "End"]
+        header_bar.remove(end_widget)
+        toolbar_contents.add(end_widget)
+
         toolbar.show()
 
     def remove_header_bar(self):
 
         """ Remove the current CSD headerbar, and show the regular titlebar """
 
+        key, mod = Gtk.accelerator_parse("F10")
+        self.HeaderMenu.remove_accelerator(self.accel_group, key, mod)
+
         self.MainWindow.unrealize()
         self.MainWindow.set_titlebar(None)
         self.MainWindow.map()
@@ -1359,13 +1370,10 @@ class NicotineFrame:
         toolbar_contents = self.__dict__[self.current_page_id + "ToolbarContents"]
 
         title_widget = self.__dict__[self.current_page_id + "Title"]
+        title_widget.set_hexpand(False)
         toolbar_contents.remove(title_widget)
         header_bar.set_custom_title(title_widget)
 
-        end_widget = self.__dict__[self.current_page_id + "End"]
-        toolbar_contents.remove(end_widget)
-        header_bar.pack_end(end_widget)
-
         try:
             start_widget = self.__dict__[self.current_page_id + "Start"]
             toolbar_contents.remove(start_widget)
@@ -1375,6 +1383,10 @@ class NicotineFrame:
             # No start widget
             pass
 
+        end_widget = self.__dict__[self.current_page_id + "End"]
+        toolbar_contents.remove(end_widget)
+        header_bar.pack_end(end_widget)
+
         toolbar.hide()
 
     def set_active_header_bar(self, page_id):
@@ -1884,11 +1896,11 @@ class NicotineFrame:
                 else:
                     self.on_browse_buddy_shares(folder=folder)
             else:
+                self.userbrowse.show_user(user, folder=folder)
+
                 if self.userbrowse.is_new_request(user):
                     self.np.send_message_to_peer(user, slskmessages.GetSharedFileList(None))
 
-                self.userbrowse.show_user(user, folder=folder)
-
     def parse_local_shares(self, username, msg, folder=None, shares_type="normal"):
         """ Parse our local shares list and show it in the UI """
 
@@ -1978,7 +1990,7 @@ class NicotineFrame:
         if self.np.config.sections["words"]["replacewords"]:
             autoreplaced = self.np.config.sections["words"]["autoreplaced"]
             for word, replacement in autoreplaced.items():
-                message = message.replace(word, replacement)
+                message = message.replace(str(word), str(replacement))
 
         return message
 
@@ -1987,6 +1999,7 @@ class NicotineFrame:
             filler = self.np.config.sections["words"]["censorfill"]
             censored = self.np.config.sections["words"]["censored"]
             for word in censored:
+                word = str(word)
                 message = message.replace(word, filler * len(word))
 
         return message
@@ -2068,7 +2081,8 @@ class NicotineFrame:
     def on_auto_away(self):
         if not self.away:
             self.autoaway = True
-            self.on_away()
+            self.away = True
+            self._apply_away_state()
 
         return False
 
@@ -2078,7 +2092,8 @@ class NicotineFrame:
 
             if self.away:
                 # Disable away mode if not already done
-                self.on_away()
+                self.away = False
+                self._apply_away_state()
 
         if self.awaytimerid is not None:
             self.remove_away_timer(self.awaytimerid)
@@ -2279,25 +2294,13 @@ class NicotineFrame:
         else:
             return self.images["offline"]
 
-    def has_user_flag(self, user, flag):
-        if flag.lower() not in self.flag_images:
-            self.get_flag_image(flag)
+    def has_user_flag(self, user, country):
 
-        if flag.lower() not in self.flag_images:
+        if not self.get_flag_image(country):
             return
 
-        self.flag_users[user] = flag
-        self.chatrooms.set_user_flag(user, flag)
-        self.userlist.set_user_flag(user, flag)
-
-    def get_user_flag(self, user):
-        if user not in self.flag_users:
-            for i in self.np.config.sections["server"]["userlist"]:
-                if user == i[0] and i[6] is not None:
-                    return i[6]
-            return "flag_"
-        else:
-            return self.flag_users[user]
+        self.chatrooms.set_user_flag(user, country)
+        self.userlist.set_user_flag(user, country)
 
     def on_settings_downloads(self, widget):
         self.on_settings(page='Downloads')
@@ -2369,8 +2372,8 @@ class NicotineFrame:
         log_path = self.np.config.sections["logging"]["debuglogsdir"]
 
         try:
-            if not os.path.exists(log_path):
-                os.mkdir(log_path)
+            if not os.path.isdir(log_path):
+                os.makedirs(log_path)
 
             open_file_path(self.np.config.sections["logging"]["debuglogsdir"])
 
@@ -2715,6 +2718,13 @@ class NicotineFrame:
         self.shutdown = True
         self.np.manualdisconnect = True
 
+        # Notify plugins
+        self.np.pluginhandler.shutdown_notification()
+
+        # Disable plugins
+        for plugin in self.np.pluginhandler.list_installed_plugins():
+            self.np.pluginhandler.disable_plugin(plugin)
+
         # Shut down networking thread
         server_conn = self.np.active_server_conn
 
