@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
 # COPYRIGHT (C) 2003-2004 Hyriand <hyriand@thegraveyard.org>
 #
@@ -20,8 +20,6 @@
 
 import os
 
-from gettext import gettext as _
-
 from gi.repository import Gdk
 from gi.repository import Gtk
 from gi.repository import GObject
@@ -32,6 +30,8 @@ from pynicotine.gtkgui.utils import human_speed
 from pynicotine.gtkgui.utils import initialise_columns
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import set_treeview_selected_row
+from pynicotine.gtkgui.utils import triggers_context_menu
 from pynicotine.gtkgui.utils import update_widget_visuals
 
 
@@ -43,60 +43,60 @@ class Interests:
         self.np = np
 
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "interests.ui"))
+        self.frame.interestsvbox.add(self.Main)
 
         self.likes = {}
         self.likes_model = Gtk.ListStore(str)
         self.likes_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         cols = initialise_columns(
+            None,
             self.LikesList,
-            [_("I like") + ":", 0, "text"]
+            ["i_like", _("I Like"), -1, "text", None]
         )
 
-        cols[0].set_sort_column_id(0)
+        cols["i_like"].set_sort_column_id(0)
         self.LikesList.set_model(self.likes_model)
 
         self.til_popup_menu = popup = PopupMenu(self.frame)
 
         popup.setup(
-            ("#" + _("_Remove this item"), self.on_remove_thing_i_like),
-            ("#" + _("Re_commendations for this item"), self.on_recommend_item),
+            ("#" + _("_Remove Item"), self.on_remove_thing_i_like),
+            ("#" + _("Re_commendations For Item"), self.on_recommend_item),
             ("", None),
-            ("#" + _("_Search for this item"), self.on_recommend_search)
+            ("#" + _("_Search For Item"), self.on_recommend_search)
         )
 
-        self.LikesList.connect("button_press_event", self.on_popup_til_menu)
-
         self.dislikes = {}
         self.dislikes_model = Gtk.ListStore(str)
         self.dislikes_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         cols = initialise_columns(
+            None,
             self.DislikesList,
-            [_("I dislike") + ":", 0, "text"]
+            ["i_dislike", _("I Dislike"), -1, "text", None]
         )
 
-        cols[0].set_sort_column_id(0)
+        cols["i_dislike"].set_sort_column_id(0)
         self.DislikesList.set_model(self.dislikes_model)
 
         self.tidl_popup_menu = popup = PopupMenu(self.frame)
 
         popup.setup(
-            ("#" + _("_Remove this item"), self.on_remove_thing_i_dislike),
+            ("#" + _("_Remove Item"), self.on_remove_thing_i_dislike),
             ("", None),
-            ("#" + _("_Search for this item"), self.on_recommend_search)
+            ("#" + _("_Search For Item"), self.on_recommend_search)
         )
 
-        self.DislikesList.connect("button_press_event", self.on_popup_tidl_menu)
-
         cols = initialise_columns(
+            None,
             self.RecommendationsList,
-            [_("Rating"), 0, "text"],
-            [_("Item"), -1, "text"]
+            ["rating", _("Rating"), 0, "number", None],
+            ["item", _("Item"), -1, "text", None]
         )
 
-        cols[0].set_sort_column_id(2)
-        cols[1].set_sort_column_id(1)
+        cols["rating"].set_sort_column_id(2)
+        cols["item"].set_sort_column_id(1)
 
         self.recommendations_model = Gtk.ListStore(
             str,  # (0) hrating
@@ -108,23 +108,22 @@ class Interests:
         self.r_popup_menu = popup = PopupMenu(self.frame)
 
         popup.setup(
-            ("$" + _("I _like this"), self.on_like_recommendation),
-            ("$" + _("I _don't like this"), self.on_dislike_recommendation),
-            ("#" + _("_Recommendations for this item"), self.on_recommend_recommendation),
+            ("$" + _("I _Like This"), self.on_like_recommendation),
+            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
+            ("#" + _("_Recommendations For Item"), self.on_recommend_recommendation),
             ("", None),
-            ("#" + _("_Search for this item"), self.on_recommend_search)
+            ("#" + _("_Search For Item"), self.on_recommend_search)
         )
 
-        self.RecommendationsList.connect("button_press_event", self.on_popup_r_menu)
-
         cols = initialise_columns(
+            None,
             self.UnrecommendationsList,
-            [_("Rating"), 0, "text"],
-            [_("Item"), -1, "text"]
+            ["rating", _("Rating"), 0, "number", None],
+            ["item", _("Item"), -1, "text", None]
         )
 
-        cols[0].set_sort_column_id(2)
-        cols[1].set_sort_column_id(1)
+        cols["rating"].set_sort_column_id(2)
+        cols["item"].set_sort_column_id(1)
 
         self.unrecommendations_model = Gtk.ListStore(
             str,  # (0) hrating
@@ -136,27 +135,28 @@ class Interests:
         self.ur_popup_menu = popup = PopupMenu(self.frame)
 
         popup.setup(
-            ("$" + _("I _like this"), self.on_like_recommendation),
-            ("$" + _("I _don't like this"), self.on_dislike_recommendation),
-            ("#" + _("_Recommendations for this item"), self.on_recommend_recommendation),
+            ("$" + _("I _Like This"), self.on_like_recommendation),
+            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
+            ("#" + _("_Recommendations For Item"), self.on_recommend_recommendation),
             ("", None),
-            ("#" + _("_Search for this item"), self.on_recommend_search)
+            ("#" + _("_Search For Item"), self.on_recommend_search)
         )
 
-        self.UnrecommendationsList.connect("button_press_event", self.on_popup_un_rec_menu)
-
         cols = initialise_columns(
+            None,
             self.RecommendationUsersList,
-            ["", 25, "pixbuf"],
-            [_("User"), 100, "text"],
-            [_("Speed"), 0, "text"],
-            [_("Files"), 0, "text"],
+            ["country", _("Country"), 25, "pixbuf", None],
+            ["user", _("User"), 100, "text", None],
+            ["speed", _("Speed"), 100, "text", None],
+            ["files", _("Files"), 100, "text", None],
         )
 
-        cols[0].set_sort_column_id(4)
-        cols[1].set_sort_column_id(1)
-        cols[2].set_sort_column_id(5)
-        cols[3].set_sort_column_id(6)
+        cols["country"].set_sort_column_id(4)
+        cols["user"].set_sort_column_id(1)
+        cols["speed"].set_sort_column_id(5)
+        cols["files"].set_sort_column_id(6)
+
+        cols["country"].get_widget().hide()
 
         self.recommendation_users = {}
         self.recommendation_users_model = Gtk.ListStore(
@@ -172,32 +172,21 @@ class Interests:
         self.recommendation_users_model.set_sort_column_id(1, Gtk.SortType.ASCENDING)
 
         self.ru_popup_menu = popup = PopupMenu(self.frame)
-        popup.setup(
-            ("#" + _("Send _message"), popup.on_send_message),
-            ("", None),
-            ("#" + _("Show IP a_ddress"), popup.on_show_ip_address),
-            ("#" + _("Get user i_nfo"), popup.on_get_user_info),
-            ("#" + _("Brow_se files"), popup.on_browse_user),
-            ("#" + _("Gi_ve privileges"), popup.on_give_privileges),
-            ("", None),
-            ("$" + _("_Add user to list"), popup.on_add_to_list),
-            ("$" + _("_Ban this user"), popup.on_ban_user),
-            ("$" + _("_Ignore this user"), popup.on_ignore_user)
-        )
-
-        self.RecommendationUsersList.connect("button_press_event", self.on_popup_ru_menu)
+        popup.setup_user_menu()
 
         for thing in self.np.config.sections["interests"]["likes"]:
-            self.likes[thing] = self.likes_model.append([thing])
+            if thing and isinstance(thing, str):
+                self.likes[thing] = self.likes_model.append([thing])
 
         for thing in self.np.config.sections["interests"]["dislikes"]:
-            self.dislikes[thing] = self.dislikes_model.append([thing])
+            if thing and isinstance(thing, str):
+                self.dislikes[thing] = self.dislikes_model.append([thing])
 
         self.update_visuals()
 
-    def on_add_thing_i_like(self, widget):
-        thing = self.AddLikeEntry.get_text()
-        self.AddLikeEntry.set_text("")
+    def on_add_thing_i_like(self, widget, *args):
+        thing = widget.get_text()
+        widget.set_text("")
 
         if thing and thing.lower() not in self.np.config.sections["interests"]["likes"]:
             thing = thing.lower()
@@ -206,9 +195,9 @@ class Interests:
             self.np.config.write_configuration()
             self.np.queue.put(slskmessages.AddThingILike(thing))
 
-    def on_add_thing_i_dislike(self, widget):
-        thing = self.AddDislikeEntry.get_text()
-        self.AddDislikeEntry.set_text("")
+    def on_add_thing_i_dislike(self, widget, *args):
+        thing = widget.get_text()
+        widget.set_text("")
 
         if thing and thing.lower() not in self.np.config.sections["interests"]["dislikes"]:
             thing = thing.lower()
@@ -246,14 +235,16 @@ class Interests:
     def on_like_recommendation(self, widget):
         thing = widget.get_parent().get_user()
 
-        if widget.get_active() and thing not in self.np.config.sections["interests"]["likes"]:
+        if widget.get_active() and \
+                thing and thing not in self.np.config.sections["interests"]["likes"]:
             self.np.config.sections["interests"]["likes"].append(thing)
             self.likes[thing] = self.likes_model.append([thing])
 
             self.np.config.write_configuration()
             self.np.queue.put(slskmessages.AddThingILike(thing))
 
-        elif not widget.get_active() and thing in self.np.config.sections["interests"]["likes"]:
+        elif not widget.get_active() and \
+                thing and thing in self.np.config.sections["interests"]["likes"]:
             self.likes_model.remove(self.likes[thing])
             del self.likes[thing]
             self.np.config.sections["interests"]["likes"].remove(thing)
@@ -264,14 +255,16 @@ class Interests:
     def on_dislike_recommendation(self, widget):
         thing = widget.get_parent().get_user()
 
-        if widget.get_active() and thing not in self.np.config.sections["interests"]["dislikes"]:
+        if widget.get_active() and \
+                thing and thing not in self.np.config.sections["interests"]["dislikes"]:
             self.np.config.sections["interests"]["dislikes"].append(thing)
             self.dislikes[thing] = self.dislikes_model.append([thing])
 
             self.np.config.write_configuration()
             self.np.queue.put(slskmessages.AddThingIHate(thing))
 
-        elif not widget.get_active() and thing in self.np.config.sections["interests"]["dislikes"]:
+        elif not widget.get_active() and \
+                thing and thing in self.np.config.sections["interests"]["dislikes"]:
             self.dislikes_model.remove(self.dislikes[thing])
             del self.dislikes[thing]
             self.np.config.sections["interests"]["dislikes"].remove(thing)
@@ -291,7 +284,7 @@ class Interests:
 
     def on_recommend_search(self, widget):
         thing = widget.get_parent().get_user()
-        self.frame.search_entry.set_text(thing)
+        self.frame.SearchEntry.set_text(thing)
         self.frame.change_main_page("search")
 
     def on_global_recommendations_clicked(self, widget):
@@ -353,100 +346,126 @@ class Interests:
 
         self.recommendation_users_model.set(self.recommendation_users[msg.user], 2, human_speed(msg.avgspeed), 3, humanize(msg.files), 5, msg.avgspeed, 6, msg.files)
 
-    def on_popup_ru_menu(self, widget, event):
-        items = self.ru_popup_menu.get_children()
-        d = self.RecommendationUsersList.get_path_at_pos(int(event.x), int(event.y))
+    def get_selected_item(self, treeview, column=0):
 
-        if not d:
-            return
+        model, iterator = treeview.get_selection().get_selected()
 
-        path, column, x, y = d
-        user = self.recommendation_users_model.get_value(self.recommendation_users_model.get_iter(path), 1)
+        if iterator is None:
+            return None
 
-        if event.button != 3:
-            if event.type == Gdk.EventType._2BUTTON_PRESS:
-                self.privatechats.send_message(user)
-                self.change_main_page("private")
-            return
+        return model.get_value(iterator, column)
+
+    def on_ru_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_ru_menu(widget)
+
+        if event.type == Gdk.EventType._2BUTTON_PRESS:
+            user = self.get_selected_item(widget)
+
+            if user is not None:
+                self.frame.privatechats.send_message(user)
+                self.frame.change_main_page("private")
+                return True
+
+        return False
+
+    def on_popup_ru_menu(self, widget):
+
+        user = self.get_selected_item(widget, column=1)
+        if user is None:
+            return False
 
         self.ru_popup_menu.set_user(user)
-        items[7].set_active(user in (i[0] for i in self.np.config.sections["server"]["userlist"]))
-        items[8].set_active(user in self.np.config.sections["server"]["banlist"])
-        items[9].set_active(user in self.np.config.sections["server"]["ignorelist"])
+        self.ru_popup_menu.toggle_user_items()
 
-        self.ru_popup_menu.popup(None, None, None, None, event.button, event.time)
+        self.ru_popup_menu.popup()
+        return True
 
-    def on_popup_til_menu(self, widget, event):
-        if event.button != 3:
-            return
+    def on_til_list_clicked(self, widget, event):
 
-        d = self.LikesList.get_path_at_pos(int(event.x), int(event.y))
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_til_menu(widget)
 
-        if not d:
-            return
+        return False
 
-        path, column, x, y = d
-        iterator = self.likes_model.get_iter(path)
-        thing = self.likes_model.get_value(iterator, 0)
+    def on_popup_til_menu(self, widget):
 
-        self.til_popup_menu.set_user(thing)
-        self.til_popup_menu.popup(None, None, None, None, event.button, event.time)
+        item = self.get_selected_item(widget, column=0)
+        if item is None:
+            return False
 
-    def on_popup_tidl_menu(self, widget, event):
-        if event.button != 3:
-            return
+        self.til_popup_menu.set_user(item)
 
-        d = self.DislikesList.get_path_at_pos(int(event.x), int(event.y))
+        self.til_popup_menu.popup()
+        return True
 
-        if not d:
-            return
+    def on_tidl_list_clicked(self, widget, event):
 
-        path, column, x, y = d
-        iterator = self.dislikes_model.get_iter(path)
-        thing = self.dislikes_model.get_value(iterator, 0)
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_tidl_menu(widget)
 
-        self.tidl_popup_menu.set_user(thing)
-        self.tidl_popup_menu.popup(None, None, None, None, event.button, event.time)
+        return False
 
-    def on_popup_r_menu(self, widget, event):
-        if event.button != 3:
-            return
+    def on_popup_tidl_menu(self, widget):
 
-        d = self.RecommendationsList.get_path_at_pos(int(event.x), int(event.y))
+        item = self.get_selected_item(widget, column=0)
+        if item is None:
+            return False
 
-        if not d:
-            return
+        self.tidl_popup_menu.set_user(item)
 
-        path, column, x, y = d
-        iterator = self.recommendations_model.get_iter(path)
-        thing = self.recommendations_model.get_value(iterator, 0)
-        items = self.r_popup_menu.get_children()
+        self.tidl_popup_menu.popup()
+        return True
 
-        self.r_popup_menu.set_user(thing)
-        items[0].set_active(thing in self.np.config.sections["interests"]["likes"])
-        items[1].set_active(thing in self.np.config.sections["interests"]["dislikes"])
+    def on_r_list_clicked(self, widget, event):
 
-        self.r_popup_menu.popup(None, None, None, None, event.button, event.time)
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_r_menu(widget)
 
-    def on_popup_un_rec_menu(self, widget, event):
-        if event.button != 3:
-            return
+        return False
 
-        d = self.UnrecommendationsList.get_path_at_pos(int(event.x), int(event.y))
+    def on_popup_r_menu(self, widget):
 
-        if not d:
-            return
+        item = self.get_selected_item(widget, column=1)
+        if item is None:
+            return False
 
-        path, column, x, y = d
-        iterator = self.unrecommendations_model.get_iter(path)
-        thing = self.unrecommendations_model.get_value(iterator, 0)
-        items = self.ur_popup_menu.get_children()
+        self.r_popup_menu.set_user(item)
 
-        self.ur_popup_menu.set_user(thing)
-        items[0].set_active(thing in self.np.config.sections["interests"]["likes"])
-        items[1].set_active(thing in self.np.config.sections["interests"]["dislikes"])
+        items = self.r_popup_menu.get_items()
+        items[_("I _Like This")].set_active(item in self.np.config.sections["interests"]["likes"])
+        items[_("I _Dislike This")].set_active(item in self.np.config.sections["interests"]["dislikes"])
 
-        self.ur_popup_menu.popup(None, None, None, None, event.button, event.time)
+        self.r_popup_menu.popup()
+        return True
+
+    def on_un_rec_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_un_rec_menu(widget)
+
+        return False
+
+    def on_popup_un_rec_menu(self, widget):
+
+        item = self.get_selected_item(widget, column=1)
+        if item is None:
+            return False
+
+        self.ur_popup_menu.set_user(item)
+
+        items = self.ur_popup_menu.get_items()
+        items[_("I _Like This")].set_active(item in self.np.config.sections["interests"]["likes"])
+        items[_("I _Dislike This")].set_active(item in self.np.config.sections["interests"]["dislikes"])
+
+        self.ur_popup_menu.popup()
+        return True
 
     def update_visuals(self):
 
