@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -15,45 +15,24 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-import gi
+
 import sys
 import _thread
 
-from gettext import gettext as _
-
+from gi.repository import Gdk
+from gi.repository import Gio
 from gi.repository import GLib
 
 from pynicotine.logfacility import log
 from pynicotine.utils import execute_command
-from pynicotine.utils import version
 
 
 class Notifications:
 
     def __init__(self, frame):
 
-        self.use_libnotify = False
-        self.use_plyer = False
-
-        try:
-            # Notification support
-            gi.require_version('Notify', '0.7')
-            from gi.repository import Notify
-            Notify.init("Nicotine+")
-            self.notification_provider = Notify
-            self.use_libnotify = True
-
-        except (ImportError, ValueError):
-            try:
-                # Windows support via plyer
-                from plyer import notification
-                self.notification_provider = notification
-                self.use_plyer = True
-
-            except (ImportError, ValueError):
-                self.notification_provider = None
-
         self.frame = frame
+        self.application = Gio.Application.get_default()
         self.tts = []
         self.tts_playing = False
         self.continue_playing = False
@@ -64,17 +43,20 @@ class Notifications:
             if room not in self.frame.hilites["rooms"]:
                 self.frame.hilites["rooms"].append(room)
 
-                self.frame.tray_app.set_image()
+                self.frame.tray.set_image()
+
         elif location == "private":
             if user in self.frame.hilites[location]:
                 self.frame.hilites[location].remove(user)
                 self.frame.hilites[location].append(user)
+
             elif user not in self.frame.hilites[location]:
                 self.frame.hilites[location].append(user)
 
-                self.frame.tray_app.set_image()
+                self.frame.tray.set_image()
 
-        if tab and self.frame.np.config.sections["ui"]["urgencyhint"] and not self.frame.got_focus:
+        if tab and self.frame.np.config.sections["ui"]["urgencyhint"] and \
+                not self.frame.MainWindow.is_focus():
             self.frame.MainWindow.set_urgency_hint(True)
 
         self.set_title(user)
@@ -102,31 +84,33 @@ class Notifications:
                 self.frame.hilites["private"].remove(user)
             self.set_title(user)
 
-        self.frame.tray_app.set_image()
+        self.frame.tray.set_image()
 
     def set_title(self, user=None):
 
+        app_name = GLib.get_application_name()
+
         if self.frame.hilites["rooms"] == [] and self.frame.hilites["private"] == []:
             # Reset Title
-            if self.frame.MainWindow.get_title() != "Nicotine+" + " " + version:
-                self.frame.MainWindow.set_title("Nicotine+" + " " + version)
+            self.frame.MainWindow.set_title(app_name)
+
         elif self.frame.np.config.sections["notifications"]["notification_window_title"]:
             # Private Chats have a higher priority
             if len(self.frame.hilites["private"]) > 0:
                 user = self.frame.hilites["private"][-1]
                 self.frame.MainWindow.set_title(
-                    "Nicotine+" + " " + version + " :: " + _("Private Message from %(user)s") % {'user': user}
+                    app_name + " - " + _("Private Message from %(user)s") % {'user': user}
                 )
             # Allow for the possibility the username is not available
             elif len(self.frame.hilites["rooms"]) > 0:
                 room = self.frame.hilites["rooms"][-1]
                 if user is None:
                     self.frame.MainWindow.set_title(
-                        "Nicotine+" + " " + version + " :: " + _("You've been mentioned in the %(room)s room") % {'room': room}
+                        app_name + " - " + _("You've been mentioned in the %(room)s room") % {'room': room}
                     )
                 else:
                     self.frame.MainWindow.set_title(
-                        "Nicotine+" + " " + version + " :: " + _("%(user)s mentioned you in the %(room)s room") % {'user': user, 'room': room}
+                        app_name + " - " + _("%(user)s mentioned you in the %(room)s room") % {'user': user, 'room': room}
                     )
 
     def new_tts(self, message):
@@ -167,30 +151,42 @@ class Notifications:
 
         execute_command(self.frame.np.config.sections["ui"]["speechcommand"], message)
 
-    def new_notification(self, message, title="Nicotine+", soundnamenotify="message-sent-instant", soundnamewin="SystemAsterisk"):
+    def new_notification(self, message, title=None, priority=Gio.NotificationPriority.NORMAL):
+
+        if title is None:
+            title = GLib.get_application_name()
 
         try:
-            if self.use_libnotify:
-                notification_popup = self.notification_provider.Notification.new(title, message)
-                notification_popup.set_hint("desktop-entry", GLib.Variant("s", "org.nicotine_plus.Nicotine"))
+            if sys.platform == "win32":
 
-                if self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
-                    notification_popup.set_hint("sound-name", GLib.Variant("s", soundnamenotify))
+                """ Windows notification popup support via plyer
+                Once https://gitlab.gnome.org/GNOME/glib/-/issues/1234 is implemented, we can drop plyer """
 
-                notification_popup.set_image_from_pixbuf(self.frame.images["notify"])
+                from plyer import notification
 
-                notification_popup.show()
-
-            elif self.use_plyer:
-                self.notification_provider.notify(
-                    app_name="Nicotine+",
+                notification.notify(
+                    app_name=GLib.get_application_name(),
                     title=title,
                     message=message
                 )
 
-                if sys.platform == "win32" and self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
+                if self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
                     import winsound
-                    winsound.PlaySound(soundnamewin, winsound.SND_ALIAS)
+                    winsound.PlaySound("SystemAsterisk", winsound.SND_ALIAS)
+                return
+
+            notification_popup = Gio.Notification.new(title)
+            notification_popup.set_body(message)
+
+            if self.frame.images["notify"]:
+                notification_popup.set_icon(self.frame.images["notify"])
+
+            notification_popup.set_priority(priority)
+
+            self.application.send_notification(None, notification_popup)
+
+            if self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
+                Gdk.beep()
 
         except Exception as error:
             log.add(_("Unable to show notification popup: %s"), str(error))
