@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -18,10 +18,10 @@
 
 import os
 
-from gi.repository import Gtk
-
 from pynicotine import slskmessages
 from pynicotine.gtkgui.utils import append_line
+from pynicotine.gtkgui.utils import load_ui_elements
+from pynicotine.gtkgui.utils import update_widget_visuals
 
 
 class RoomWall:
@@ -31,29 +31,13 @@ class RoomWall:
         self.frame = frame
         self.room = room
 
-        builder = Gtk.Builder()
-        builder.set_translation_domain('nicotine')
-        builder.add_from_file(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ui", "roomwall.ui"))
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "dialogs", "roomwall.ui"))
+        self.RoomWallDialog.set_transient_for(frame.MainWindow)
 
-        self.room_wall_dialog = builder.get_object("RoomWallDialog")
-        builder.connect_signals(self)
+    def on_set_room_wall_message(self, widget, *args):
 
-        for i in builder.get_objects():
-            try:
-                self.__dict__[Gtk.Buildable.get_name(i)] = i
-            except TypeError:
-                pass
-
-        self.room_wall_dialog.set_transient_for(frame.MainWindow)
-
-        self.room_wall_dialog.connect("destroy", self.hide)
-        self.room_wall_dialog.connect("destroy-event", self.hide)
-        self.room_wall_dialog.connect("delete-event", self.hide)
-        self.room_wall_dialog.connect("delete_event", self.hide)
-
-    def on_set_room_wall_message(self, widget):
-        result = self.RoomWallEntry.get_text()
-        self.RoomWallEntry.set_text("")
+        result = widget.get_text()
+        widget.set_text("")
 
         config = self.frame.np.config.sections
         room_name = self.room.room
@@ -70,15 +54,22 @@ class RoomWall:
         tickers = self.room.tickers.get_tickers()
         append_line(self.RoomWallList, "%s" % ("\n".join(["[%s] %s" % (user, msg) for (user, msg) in tickers if not user == login])), showstamp=False, scroll=False)
 
-    def hide(self, w=None, event=None):
+    def update_visuals(self):
+
+        for widget in self.__dict__.values():
+            update_widget_visuals(widget)
+
+    def hide(self, widget=None, event=None):
+
         self.RoomWallList.get_buffer().set_text("")
-        self.room_wall_dialog.hide()
+        self.RoomWallDialog.hide()
         return True
 
     def show(self):
+
         tickers = self.room.tickers.get_tickers()
         append_line(self.RoomWallList, "%s" % ("\n".join(["[%s] %s" % (user, msg) for (user, msg) in tickers])), showstamp=False, scroll=False)
-        self.room_wall_dialog.show()
+        self.RoomWallDialog.show()
 
 
 class Tickers:
