@@ -26,7 +26,6 @@ import os
 
 from gi.repository import Gdk
 
-from pynicotine import slskmessages
 from pynicotine.gtkgui.dialogs import option_dialog
 from pynicotine.gtkgui.transferlist import TransferList
 from pynicotine.gtkgui.utils import open_file_path
@@ -192,11 +191,11 @@ class Uploads(TransferList):
         self.frame.np.transfers.calc_upload_queue_sizes()
         self.frame.np.transfers.check_upload_queue()
 
-    def on_abort_transfer(self, widget, remove_file=False, clear=False):
+    def on_abort_transfer(self, widget, clear=False):
 
         self.select_transfers()
 
-        self.abort_transfers(remove_file, clear)
+        self.abort_transfers(clear)
 
         self.frame.np.transfers.calc_upload_queue_sizes()
         self.frame.np.transfers.check_upload_queue()
@@ -233,14 +232,6 @@ class Uploads(TransferList):
         self.select_transfers()
 
         for transfer in self.selected_transfers:
-            filename = transfer.filename
-            path = transfer.path
-            user = transfer.user
-
-            if user in self.frame.np.transfers.get_transferring_users():
-                continue
-
-            self.frame.np.send_message_to_peer(user, slskmessages.UploadQueueNotification(None))
-            self.frame.np.transfers.push_file(user, filename, path, transfer=transfer)
+            self.frame.np.transfers.retry_upload(transfer)
 
         self.frame.np.transfers.check_upload_queue()
