@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2013 SeeSchloss <see@seos.fr>
 # COPYRIGHT (C) 2009-2010 Quinox <quinox@users.sf.net>
@@ -22,7 +22,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from gettext import gettext as _
+
 from sys import maxsize
 
 from gi.repository import Gdk
@@ -31,50 +31,50 @@ from gi.repository import Gtk
 
 from _thread import start_new_thread
 from pynicotine import slskmessages
-from pynicotine.gtkgui.dirchooser import choose_dir
+from pynicotine.gtkgui.dialogs import choose_dir
 from pynicotine.gtkgui.dialogs import combo_box_dialog
-from pynicotine.gtkgui.utils import hide_columns
+from pynicotine.gtkgui.fileproperties import FileProperties
 from pynicotine.gtkgui.utils import human_size
+from pynicotine.gtkgui.utils import InfoBar
 from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import load_ui_elements
+from pynicotine.gtkgui.utils import open_file_path
 from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import save_columns
 from pynicotine.gtkgui.utils import set_treeview_selected_row
+from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import update_widget_visuals
 from pynicotine.logfacility import log
 from pynicotine.utils import clean_file
-from pynicotine.utils import execute_command
 from pynicotine.utils import get_result_bitrate_length
 
 
 class UserBrowse:
 
-    def __init__(self, userbrowses, user, conn):
-
-        # Build the window
-        builder = Gtk.Builder()
-
-        builder.set_translation_domain('nicotine')
-        builder.add_from_file(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ui", "userbrowse.ui"))
-
-        self.user_browse_tab = builder.get_object("UserBrowseTab")
-
-        for i in builder.get_objects():
-            try:
-                self.__dict__[Gtk.Buildable.get_name(i)] = i
-            except TypeError:
-                pass
-
-        self.user_browse_tab.remove(self.Main)
-        self.user_browse_tab.destroy()
-
-        builder.connect_signals(self)
+    def __init__(self, userbrowses, user):
 
         self.userbrowses = userbrowses
-
         self.frame = userbrowses.frame
+
+        # Build the window
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "userbrowse.ui"))
+        self.info_bar = InfoBar(self.InfoBar, Gtk.MessageType.INFO)
+
+        # Monitor user online status
+        if user not in self.frame.np.watchedusers:
+            self.frame.np.queue.put(slskmessages.AddUser(user))
+
         self.user = user
-        self.conn = conn
+        self.conn = None
+        self.local_shares_type = None
+        self.refreshing = True
 
         # selected_folder is the current selected folder
         self.selected_folder = None
+
+        # queued_folder is a folder that should be opened once the share has loaded
+        self.queued_folder = None
+
         self.search_list = []
         self.query = None
         self.search_position = 0
@@ -91,137 +91,156 @@ class UserBrowse:
 
         self.dir_store = Gtk.TreeStore(str, str)
 
-        self.FolderTreeView.set_headers_visible(True)
-        self.FolderTreeView.set_enable_tree_lines(True)
-
+        self.dir_column_numbers = [0, 1]
         cols = initialise_columns(
+            None,
             self.FolderTreeView,
-            [_("Directories"), -1, "text", self.cell_data_func]  # 0
+            ["folders", _("Folders"), -1, "text", None]  # 0
         )
-        cols[0].set_sort_column_id(0)
+
+        cols["folders"].set_sort_column_id(0)
 
         self.popup_menu_users = PopupMenu(self.frame, False)
         self.popup_menu_users2 = PopupMenu(self.frame, False)
-        for menu in [self.popup_menu_users, self.popup_menu_users2]:
-            menu.setup(
-                ("#" + _("Send _message"), menu.on_send_message),
-                ("#" + _("Show IP a_ddress"), menu.on_show_ip_address),
-                ("#" + _("Get user i_nfo"), menu.on_get_user_info),
-                ("#" + _("Gi_ve privileges"), menu.on_give_privileges),
-                ("", None),
-                ("$" + _("_Add user to list"), menu.on_add_to_list),
-                ("$" + _("_Ban this user"), menu.on_ban_user),
-                ("$" + _("_Ignore this user"), menu.on_ignore_user)
-            )
+        self.popup_menu_users_tab = PopupMenu(self.frame)
+
+        for menu in (self.popup_menu_users, self.popup_menu_users2, self.popup_menu_users_tab):
+            menu.setup_user_menu(user)
+            menu.get_items()[_("Brow_se Files")].set_visible(False)
+
+            menu.append_item(("", None))
+            menu.append_item(("#" + _("_Save Shares List To Disk"), self.on_save))
+            menu.append_item(("#" + _("Close All Tabs"), menu.on_close_all_tabs, self.userbrowses))
+            menu.append_item(("#" + _("_Close Tab"), self.on_close))
 
         self.popup_menu_downloads_folders = PopupMenu(self.frame, False)
         self.popup_menu_downloads_folders.setup(
-            ("#" + _("_Download directory"), self.on_download_directory),
-            ("#" + _("Download directory _to..."), self.on_download_directory_to),
-            ("#" + _("Download _recursive"), self.on_download_directory_recursive),
-            ("#" + _("Download r_ecursive to..."), self.on_download_directory_recursive_to)
+            ("#" + _("_Download Folder"), self.on_download_directory),
+            ("#" + _("Download Folder _To..."), self.on_download_directory_to),
+            ("#" + _("Download _Recursive"), self.on_download_directory_recursive),
+            ("#" + _("Download R_ecursive To..."), self.on_download_directory_recursive_to)
         )
 
         self.popup_menu_downloads_files = PopupMenu(self.frame, False)
         self.popup_menu_downloads_files.setup(
-            ("#" + _("_Download file(s)"), self.on_download_files),
-            ("#" + _("Download _to..."), self.on_download_files_to),
+            ("#" + _("_Download File(s)"), self.on_download_files),
+            ("#" + _("Download _To..."), self.on_download_files_to),
             ("", None),
-            ("#" + _("_Download directory"), self.on_download_directory),
-            ("#" + _("Download directory _to..."), self.on_download_directory_to),
-            ("#" + _("Download _recursive"), self.on_download_directory_recursive),
-            ("#" + _("Download r_ecursive to..."), self.on_download_directory_recursive_to)
+            ("#" + _("_Download Folder"), self.on_download_directory),
+            ("#" + _("Download Folder _To..."), self.on_download_directory_to),
+            ("#" + _("Download _Recursive"), self.on_download_directory_recursive),
+            ("#" + _("Download R_ecursive To..."), self.on_download_directory_recursive_to)
         )
 
         self.popup_menu_uploads_folders = PopupMenu(self.frame, False)
         self.popup_menu_uploads_folders.setup(
-            ("#" + _("Upload Directory to..."), self.on_upload_directory_to),
-            ("#" + _("Upload Directory recursive to..."), self.on_upload_directory_recursive_to)
+            ("#" + _("Upload Folder To..."), self.on_upload_directory_to),
+            ("#" + _("Upload Folder Recursive To..."), self.on_upload_directory_recursive_to)
         )
 
         self.popup_menu_uploads_files = PopupMenu(self.frame, False)
         self.popup_menu_uploads_files.setup(
-            ("#" + _("Upload Directory to..."), self.on_upload_directory_to),
-            ("#" + _("Upload Directory recursive to..."), self.on_upload_directory_recursive_to),
-            ("#" + _("Up_load file(s)"), self.on_upload_files)
+            ("#" + _("Upload Folder To..."), self.on_upload_directory_to),
+            ("#" + _("Upload Folder Recursive To..."), self.on_upload_directory_recursive_to),
+            ("#" + _("Up_load File(s)"), self.on_upload_files)
         )
 
         self.folder_popup_menu = PopupMenu(self.frame)
-        self.folder_popup_menu.set_user(user)
 
         if user == self.frame.np.config.sections["server"]["login"]:
             self.folder_popup_menu.setup(
-                ("USERMENU", _("User"), self.popup_menu_users, self.on_popup_menu_folder_user),
+                ("#" + _("_Download Folder"), self.on_download_directory),
+                ("#" + _("Download Folder _To..."), self.on_download_directory_to),
+                ("#" + _("Download _Recursive"), self.on_download_directory_recursive),
+                ("#" + _("Download R_ecursive To..."), self.on_download_directory_recursive_to),
                 ("", None),
-                (1, _("Download"), self.popup_menu_downloads_folders, None),
-                (1, _("Upload"), self.popup_menu_uploads_folders, None),
+                ("#" + _("Upload Folder To..."), self.on_upload_directory_to),
+                ("#" + _("Upload Folder Recursive To..."), self.on_upload_directory_recursive_to),
                 ("", None),
+                ("#" + _("Open in File _Manager"), self.on_file_manager),
+                ("", None),
+                ("#" + _("Copy _Folder Path"), self.on_copy_file_path, False),
                 ("#" + _("Copy _URL"), self.on_copy_dir_url),
-                ("#" + _("Open in File Manager"), self.on_file_manager)
+                ("", None),
+                (1, _("User"), self.popup_menu_users, self.on_popup_menu_folder_user)
             )
         else:
             self.folder_popup_menu.setup(
-                ("USERMENU", _("User"), self.popup_menu_users, self.on_popup_menu_folder_user),
+                ("#" + _("_Download Folder"), self.on_download_directory),
+                ("#" + _("Download Folder _To..."), self.on_download_directory_to),
+                ("#" + _("Download _Recursive"), self.on_download_directory_recursive),
+                ("#" + _("Download R_ecursive To..."), self.on_download_directory_recursive_to),
                 ("", None),
-                (1, _("Download"), self.popup_menu_downloads_folders, None),
+                ("#" + _("Copy _Folder Path"), self.on_copy_file_path, False),
+                ("#" + _("Copy _URL"), self.on_copy_dir_url),
                 ("", None),
-                ("#" + _("Copy _URL"), self.on_copy_dir_url)
+                (1, _("User"), self.popup_menu_users, self.on_popup_menu_folder_user)
             )
 
-        self.FolderTreeView.connect("button_press_event", self.on_folder_clicked)
         self.FolderTreeView.get_selection().connect("changed", self.on_select_dir)
 
-        # Filename, HSize, Bitrate, HLength, Size, Length, RawFilename
-        self.file_store = Gtk.ListStore(str, str, str, str, GObject.TYPE_INT64, int, str)
+        self.file_store = Gtk.ListStore(
+            str,                  # (0) file name
+            str,                  # (1) hsize
+            str,                  # (2) hbitrate
+            str,                  # (3) hlength
+            GObject.TYPE_UINT64,  # (4) size
+            GObject.TYPE_UINT64,  # (5) bitrate
+            GObject.TYPE_UINT64   # (6) length
+        )
 
         self.FileTreeView.set_model(self.file_store)
-        widths = self.frame.np.config.sections["columns"]["userbrowse_widths"]
+
+        self.file_column_numbers = [0, 1, 2, 3, 4, 5, 6]
         cols = initialise_columns(
+            "user_browse",
             self.FileTreeView,
-            [_("Filename"), widths[0], "text", self.cell_data_func],
-            [_("Size"), widths[1], "number", self.cell_data_func],
-            [_("Bitrate"), widths[2], "number", self.cell_data_func],
-            [_("Length"), widths[3], "number", self.cell_data_func]
+            ["filename", _("Filename"), 600, "text", None],
+            ["size", _("Size"), 100, "number", None],
+            ["bitrate", _("Bitrate"), 100, "number", None],
+            ["length", _("Length"), 100, "number", None]
         )
-        cols[0].set_sort_column_id(0)
-        cols[1].set_sort_column_id(4)
-        cols[2].set_sort_column_id(2)
-        cols[3].set_sort_column_id(5)
+        cols["filename"].set_sort_column_id(0)
+        cols["size"].set_sort_column_id(4)
+        cols["bitrate"].set_sort_column_id(5)
+        cols["length"].set_sort_column_id(6)
         self.file_store.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
-        hide_columns(cols, self.frame.np.config.sections["columns"]["userbrowse"])
-
-        self.FileTreeView.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
-        self.FileTreeView.set_headers_clickable(True)
-        self.FileTreeView.set_rubber_banding(True)
-
         self.file_popup_menu = PopupMenu(self.frame)
 
         if user == self.frame.np.config.sections["server"]["login"]:
             self.file_popup_menu.setup(
-                ("USERMENU", "User", self.popup_menu_users2, self.on_popup_menu_file_user),
+                ("#" + "selected_files", None),
                 ("", None),
                 (1, _("Download"), self.popup_menu_downloads_files, None),
                 (1, _("Upload"), self.popup_menu_uploads_files, None),
                 ("", None),
+                ("#" + _("Send to _Player"), self.on_play_files),
+                ("#" + _("Open in File _Manager"), self.on_file_manager),
+                ("#" + _("File _Properties"), self.on_file_properties),
+                ("", None),
+                ("#" + _("Copy _File Path"), self.on_copy_file_path, True),
                 ("#" + _("Copy _URL"), self.on_copy_url),
-                ("#" + _("Send to _player"), self.on_play_files),
-                ("#" + _("Open in File Manager"), self.on_file_manager)
+                ("", None),
+                (1, "User", self.popup_menu_users2, self.on_popup_menu_file_user)
             )
         else:
             self.file_popup_menu.setup(
-                ("USERMENU", "User", self.popup_menu_users2, self.on_popup_menu_file_user),
+                ("#" + "selected_files", None),
                 ("", None),
                 (1, _("Download"), self.popup_menu_downloads_files, None),
                 ("", None),
-                ("#" + _("Copy _URL"), self.on_copy_url)
+                ("#" + _("File _Properties"), self.on_file_properties),
+                ("", None),
+                ("#" + _("Copy _File Path"), self.on_copy_file_path, True),
+                ("#" + _("Copy _URL"), self.on_copy_url),
+                ("", None),
+                (1, "User", self.popup_menu_users2, self.on_popup_menu_file_user)
             )
 
-        self.FileTreeView.connect("button_press_event", self.on_file_clicked)
+        self.update_visuals()
 
-        self.change_colours()
-
-        for name, object in list(self.__dict__.items()):
+        for name, object in self.__dict__.items():
             if isinstance(object, PopupMenu):
                 object.set_user(self.user)
 
@@ -232,43 +251,22 @@ class UserBrowse:
         self.on_popup_menu_users(self.popup_menu_users)
 
     def on_popup_menu_users(self, menu):
-
-        items = menu.get_children()
-
-        act = True
-        items[0].set_sensitive(act)
-        items[1].set_sensitive(act)
-        items[2].set_sensitive(act)
-
-        items[5].set_active(self.user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-        items[6].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
-        items[7].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
-
-        for i in range(3, 8):
-            items[i].set_sensitive(act)
-
+        menu.toggle_user_items()
         return True
 
-    def change_colours(self):
-        self.frame.set_text_bg(self.SearchEntry)
+    def update_visuals(self):
 
-        self.frame.change_list_font(self.FolderTreeView, self.frame.np.config.sections["ui"]["browserfont"])
-        self.frame.change_list_font(self.FileTreeView, self.frame.np.config.sections["ui"]["browserfont"])
-
-    def cell_data_func(self, column, cellrenderer, model, iterator, dummy="dummy"):
-        colour = self.frame.np.config.sections["ui"]["search"]
-        if colour == "":
-            colour = None
-        cellrenderer.set_property("foreground", colour)
+        for widget in self.__dict__.values():
+            update_widget_visuals(widget, list_font_target="browserfont")
 
     def on_expand(self, widget):
 
         if self.ExpandButton.get_active():
             self.FolderTreeView.expand_all()
-            self.expand.set_from_icon_name("list-remove-symbolic", Gtk.IconSize.BUTTON)
+            self.expand.set_from_icon_name("go-up-symbolic", Gtk.IconSize.BUTTON)
         else:
             self.FolderTreeView.collapse_all()
-            self.expand.set_from_icon_name("list-add-symbolic", Gtk.IconSize.BUTTON)
+            self.expand.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
 
             dirs = sorted(self.directories.keys())
 
@@ -279,87 +277,70 @@ class UserBrowse:
 
     def on_folder_clicked(self, widget, event):
 
-        pathinfo = widget.get_path_at_pos(event.x, event.y)
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_folder_popup_menu(widget)
 
-        if pathinfo is not None:
-
-            if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+            if self.user != self.frame.np.config.sections["server"]["login"]:
                 self.on_download_directory(widget)
                 return True
 
-            elif event.button == 3:
-                return self.on_folder_popup_menu(widget, event)
-
         return False
 
-    def on_folder_popup_menu(self, widget, event):
+    def on_folder_popup_menu(self, widget):
+        self.folder_popup_menu.popup()
+        return True
 
-        act = True
-        if self.selected_folder is None:
-            act = False
-
-        items = self.folder_popup_menu.get_children()
-        for item in items[1:]:
-            item.set_sensitive(act)
-
-        self.folder_popup_menu.popup(None, None, None, None, event.button, event.time)
+    def select_files(self):
+        self.selected_files = []
+        self.FileTreeView.get_selection().selected_foreach(self.selected_files_callback)
 
     def selected_files_callback(self, model, path, iterator):
-        rawfilename = self.file_store.get_value(iterator, 6)
+        rawfilename = self.file_store.get_value(iterator, 0)
         self.selected_files.append(rawfilename)
 
     def on_file_clicked(self, widget, event):
 
-        if event.button == 3:
-            return self.on_file_popup_menu(widget, event)
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_file_popup_menu(widget)
 
-        else:
-            pathinfo = widget.get_path_at_pos(event.x, event.y)
+        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+            self.select_files()
 
-            if pathinfo is None:
-                widget.get_selection().unselect_all()
-
-            elif event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
-                self.selected_files = []
-                self.FileTreeView.get_selection().selected_foreach(self.selected_files_callback)
+            if self.user == self.frame.np.config.sections["server"]["login"]:
+                self.on_play_files(widget)
+            else:
                 self.on_download_files(widget)
-                self.FileTreeView.get_selection().unselect_all()
-                return True
+            return True
 
         return False
 
-    def on_file_popup_menu(self, widget, event):
+    def on_file_popup_menu(self, widget):
 
-        set_treeview_selected_row(widget, event)
+        self.select_files()
+        num_selected_files = len(self.selected_files)
 
-        self.selected_files = []
-        self.FileTreeView.get_selection().selected_foreach(self.selected_files_callback)
-
-        files = True
-        multiple = False
-
-        if len(self.selected_files) > 1:
-            multiple = True
-
-        if len(self.selected_files) >= 1:
+        if num_selected_files >= 1:
             files = True
         else:
             files = False
 
-        items = self.file_popup_menu.get_children()
+        items = self.file_popup_menu.get_items()
 
         if self.user == self.frame.np.config.sections["server"]["login"]:
-            items[2].set_sensitive(files)  # Downloads
-            items[3].set_sensitive(files)  # Uploads
-            items[5].set_sensitive(not multiple and files)  # Copy URL
-            items[6].set_sensitive(files)  # Send to player
+            for i in (_("Download"), _("Upload"), _("Send to _Player"), _("File _Properties"),
+                      _("Copy _File Path"), _("Copy _URL")):
+                items[i].set_sensitive(files)
         else:
-            items[2].set_sensitive(files)  # Downloads
-            items[4].set_sensitive(not multiple and files)  # Copy URL
+            for i in (_("Download"), _("File _Properties"), _("Copy _File Path"), _("Copy _URL")):
+                items[i].set_sensitive(files)
 
-        self.FileTreeView.stop_emission_by_name("button_press_event")
-        self.file_popup_menu.popup(None, None, None, None, event.button, event.time)
+        items["selected_files"].set_sensitive(False)
+        items["selected_files"].set_label(_("%s File(s) Selected") % num_selected_files)
 
+        self.file_popup_menu.popup()
         return True
 
     def make_new_model(self, list):
@@ -378,8 +359,8 @@ class UserBrowse:
                 if filedata[2] < maxsize:
                     self.totalsize += filedata[2]
 
-        self.AmountShared.set_text(_("Shared: %s") % human_size(self.totalsize))
-        self.NumDirectories.set_text(_("Dirs: %s") % len(self.shares))
+        self.AmountShared.set_text(human_size(self.totalsize))
+        self.NumDirectories.set_text(str(len(self.shares)))
 
         # Generate the directory tree and select first directory
         currentdir = self.browse_get_dirs()
@@ -391,15 +372,13 @@ class UserBrowse:
             if path is not None:
                 sel.select_path(path)
 
-        self.FolderTreeView.set_sensitive(True)
-        self.FileTreeView.set_sensitive(True)
-        self.SaveButton.set_sensitive(True)
-
         if self.ExpandButton.get_active():
             self.FolderTreeView.expand_all()
         else:
             self.FolderTreeView.collapse_all()
 
+        self.set_finished()
+
     def browse_get_dirs(self):
 
         directory = ""
@@ -452,7 +431,10 @@ class UserBrowse:
                     # Other sudirs futher down the path are attached to their parent
                     current_path = dirseparator.join([path, subdir])
 
-                self.directories[current_path] = self.dir_store.append(parent, [subdir, current_path])
+                self.directories[current_path] = self.dir_store.insert_with_values(
+                    parent, -1, self.dir_column_numbers,
+                    [subdir, current_path]
+                )
 
                 # If there are subdirs futher down the path: recurse
                 if len(dictdir[subdir]):
@@ -486,6 +468,26 @@ class UserBrowse:
 
         return directory
 
+    def browse_folder(self, folder):
+        """ Browse a specific folder in the share """
+
+        try:
+            iterator = self.directories[folder]
+        except KeyError:
+            # Folder not found
+            pass
+
+        if folder:
+            sel = self.FolderTreeView.get_selection()
+            sel.unselect_all()
+
+            path = self.dir_store.get_path(iterator)
+            self.FolderTreeView.expand_to_path(path)
+            sel.select_path(path)
+            self.FolderTreeView.scroll_to_cell(path, None, True, 0.5, 0.5)
+
+            self.queued_folder = None
+
     def set_directory(self, directory):
 
         self.selected_folder = directory
@@ -505,8 +507,6 @@ class UserBrowse:
 
         for file in files:
             # Filename, HSize, Bitrate, HLength, Size, Length, RawFilename
-            rl = 0
-
             try:
                 size = int(file[2])
 
@@ -518,13 +518,17 @@ class UserBrowse:
 
             f = [file[1], human_size(size)]
 
-            h_bitrate, bitrate, h_length = get_result_bitrate_length(size, file[4])
-            f += [h_bitrate, h_length]
-
-            f += [int(size), rl, file[1]]
+            h_bitrate, bitrate, h_length, length = get_result_bitrate_length(size, file[4])
+            f += [
+                h_bitrate,
+                h_length,
+                GObject.Value(GObject.TYPE_UINT64, int(size)),
+                GObject.Value(GObject.TYPE_UINT64, bitrate),
+                GObject.Value(GObject.TYPE_UINT64, length)
+            ]
 
             try:
-                self.files[f[0]] = self.file_store.append(f)
+                self.files[f[0]] = self.file_store.insert_with_valuesv(-1, self.file_column_numbers, f)
             except Exception as msg:
                 log.add(_("Error while attempting to display folder '%(folder)s', reported error: %(error)s"), {'folder': directory, 'error': msg})
 
@@ -533,40 +537,97 @@ class UserBrowse:
 
         try:
             if not os.path.exists(sharesdir):
-                os.mkdir(sharesdir)
+                os.makedirs(sharesdir)
+
         except Exception as msg:
             log.add(_("Can't create directory '%(folder)s', reported error: %(error)s"), {'folder': sharesdir, 'error': msg})
 
         try:
-            import pickle as mypickle
-            import bz2
-            sharesfile = bz2.BZ2File(os.path.join(sharesdir, clean_file(self.user)), 'w')
-            mypickle.dump(self.shares, sharesfile, protocol=mypickle.HIGHEST_PROTOCOL)
-            sharesfile.close()
+            filepath = os.path.join(sharesdir, clean_file(self.user))
+
+            with open(filepath, "w", encoding="utf-8") as sharesfile:
+                import json
+                json.dump(self.shares, sharesfile, ensure_ascii=False)
+
             log.add(_("Saved list of shared files for user '%(user)s' to %(dir)s"), {'user': self.user, 'dir': sharesdir})
 
         except Exception as msg:
             log.add(_("Can't save shares, '%(user)s', reported error: %(error)s"), {'user': self.user, 'error': msg})
 
     def save_columns(self):
+        save_columns("user_browse", self.FileTreeView.get_columns())
 
-        columns = []
-        widths = []
+    def show_user(self, msg, folder=None, indeterminate_progress=False, local_shares_type=None):
 
-        for column in self.FileTreeView.get_columns():
-            columns.append(column.get_visible())
-            widths.append(column.get_width())
+        self.set_in_progress(indeterminate_progress)
 
-        self.frame.np.config.sections["columns"]["userbrowse"] = columns
-        self.frame.np.config.sections["columns"]["userbrowse_widths"] = widths
+        if folder:
+            self.queued_folder = folder
 
-    def show_info(self, msg):
-        self.conn = None
-        self.make_new_model(msg.list)
+        # If this is our own share, remember if it's public or buddy
+        # (needed for refresh button)
+        if local_shares_type:
+            self.local_shares_type = local_shares_type
+
+        """ Update the list model if:
+        1. This is a new user browse tab
+        2. We're refreshing the file list
+        3. This is the list of our own shared files (local_shares_type set)
+        """
+        if self.refreshing or local_shares_type:
+            if msg is None:
+                return
+
+            self.make_new_model(msg.list)
+
+        if msg and len(msg.list) == 0:
+            self.info_bar.show_message(
+                _("User's list of shared files is empty. Either the user is not sharing anything, or they are sharing files privately.")
+            )
+
+        else:
+            self.info_bar.set_visible(False)
+            self.browse_folder(self.queued_folder)
+
+        self.set_finished()
+
+    def show_connection_error(self):
+
+        self.info_bar.show_message(
+            _("Unable to request shared files from user. Either the user is offline, you both have a closed listening port, or there's a temporary connectivity issue.")
+        )
+
+        self.set_finished()
 
     def load_shares(self, list):
         self.make_new_model(list)
 
+    def is_refreshing(self):
+        return self.refreshing
+
+    def set_in_progress(self, indeterminate_progress):
+
+        if not indeterminate_progress:
+            self.progressbar1.set_fraction(0.0)
+        else:
+            self.progressbar1.set_fraction(0.5)
+
+        self.RefreshButton.set_sensitive(False)
+
+    def set_finished(self):
+
+        # Tab notification
+        self.frame.request_tab_icon(self.frame.UserBrowseTabLabel)
+        self.userbrowses.request_changed(self.Main)
+
+        self.progressbar1.set_fraction(1.0)
+
+        self.FolderTreeView.set_sensitive(True)
+        self.FileTreeView.set_sensitive(True)
+        self.RefreshButton.set_sensitive(True)
+
+        self.refreshing = False
+
     def update_gauge(self, msg):
 
         if msg.total == 0 or msg.bytes == 0:
@@ -578,6 +639,10 @@ class UserBrowse:
 
         self.progressbar1.set_fraction(fraction)
 
+    def tab_popup(self, user):
+        self.popup_menu_users_tab.toggle_user_items()
+        return self.popup_menu_users_tab
+
     def on_select_dir(self, selection):
 
         model, iterator = selection.get_selected()
@@ -592,31 +657,34 @@ class UserBrowse:
         self.FolderTreeView.expand_to_path(path)
         self.set_directory(directory)
 
-    def on_resort(self, column, column_id):
+    def selected_results_all_data(self, model, path, iterator, data):
 
-        model = self.FileTreeView.get_model()
+        filename = model.get_value(iterator, 0)
+        fn = "\\".join([self.selected_folder, filename])
+        size = model.get_value(iterator, 1)
+        bitratestr = model.get_value(iterator, 2)
+        length = model.get_value(iterator, 3)
 
-        if model.sort_col == column_id:
-            order = model.sort_order
+        data.append({
+            "user": self.user,
+            "fn": fn,
+            "filename": filename,
+            "directory": self.selected_folder,
+            "size": size,
+            "bitrate": bitratestr,
+            "length": length,
+            "immediate": None,
+            "speed": None,
+            "country": None
+        })
 
-            if order == Gtk.SortType.ASCENDING:
-                order = Gtk.SortType.DESCENDING
-            else:
-                order = Gtk.SortType.ASCENDING
+    def on_file_properties(self, widget):
 
-            column.set_sort_order(order)
-            model.sort_order = order
-            self.FileTreeView.set_model(None)
-            model.sort()
-            self.FileTreeView.set_model(model)
-            return
+        data = []
+        self.FileTreeView.get_selection().selected_foreach(self.selected_results_all_data, data)
 
-        cols = self.FileTreeView.get_columns()
-        cols[model.sort_col].set_sort_indicator(False)
-        cols[column_id].set_sort_indicator(True)
-        model.sort_col = column_id
-
-        self.on_resort(column, column_id)
+        if data:
+            FileProperties(self.frame, data).show()
 
     def on_download_directory(self, widget):
 
@@ -653,7 +721,7 @@ class UserBrowse:
 
     def download_directory(self, folder, prefix="", recurse=0):
 
-        if folder is None:
+        if self.frame.np.transfers is None or folder is None:
             return
 
         ldir = prefix + folder.split("\\")[-1]
@@ -661,38 +729,20 @@ class UserBrowse:
         # Check if folder already exists on system
         ldir = self.frame.np.transfers.folder_destination(self.user, ldir)
 
-        for d, f in self.shares:
+        for d, files in self.shares:
 
             # Find the wanted directory
             if d != folder:
                 continue
 
-            priorityfiles = []
-            normalfiles = []
-
-            if self.frame.np.config.sections["transfers"]["prioritize"]:
-
-                for file in f:
-
-                    parts = file[1].rsplit('.', 1)
-
-                    if len(parts) == 2 and parts[1] in ['sfv', 'md5', 'nfo']:
-                        priorityfiles.append(file)
-                    else:
-                        normalfiles.append(file)
-            else:
-                normalfiles = f
-
             if self.frame.np.config.sections["transfers"]["reverseorder"]:
-                deco = [(x[1], x) for x in normalfiles]
-                deco.sort(reverse=True)
-                normalfiles = [x for junk, x in deco]
+                files.sort(key=lambda x: x[1], reverse=True)
 
-            for file in priorityfiles + normalfiles:
+            for file in files:
 
                 path = "\\".join([folder, file[1]])
                 size = file[2]
-                h_bitrate, bitrate, h_length = get_result_bitrate_length(size, file[4])
+                h_bitrate, bitrate, h_length, length = get_result_bitrate_length(size, file[4])
 
                 self.frame.np.transfers.get_file(
                     self.user,
@@ -708,11 +758,14 @@ class UserBrowse:
             return
 
         for subdir, subf in self.shares:
-            if dir in subdir and dir != subdir:
+            if folder in subdir and folder != subdir:
                 self.download_directory(subdir, os.path.join(ldir, ""))
 
     def on_download_files(self, widget, prefix=""):
 
+        if not self.frame.np.transfers:
+            return
+
         folder = self.selected_folder
 
         for d, f in self.shares:
@@ -729,7 +782,7 @@ class UserBrowse:
 
                 path = "\\".join([folder, file[1]])
                 size = file[2]
-                h_bitrate, bitrate, h_length = get_result_bitrate_length(size, file[4])
+                h_bitrate, bitrate, h_length, length = get_result_bitrate_length(size, file[4])
 
                 # Get the file
                 self.frame.np.transfers.get_file(self.user, path, prefix, size=size, bitrate=h_bitrate, length=h_length, checkduplicate=True)
@@ -773,7 +826,7 @@ class UserBrowse:
         users.sort()
         user = combo_box_dialog(
             parent=self.frame.MainWindow,
-            title=_("Upload Directory's Contents"),
+            title=_("Upload Folder's Contents"),
             message=_('Enter the User you wish to upload to:'),
             droplist=users
         )
@@ -781,7 +834,7 @@ class UserBrowse:
         if user is None or user == "":
             return
 
-        self.frame.np.process_request_to_peer(user, slskmessages.UploadQueueNotification(None))
+        self.frame.np.send_message_to_peer(user, slskmessages.UploadQueueNotification(None))
 
         self.upload_directory_to(user, folder, recurse)
 
@@ -790,6 +843,9 @@ class UserBrowse:
 
     def upload_directory_to(self, user, folder, recurse=0):
 
+        if not self.frame.np.transfers:
+            return
+
         if folder == "" or folder is None or user is None or user == "":
             return
 
@@ -818,6 +874,9 @@ class UserBrowse:
 
     def on_upload_files(self, widget, prefix=""):
 
+        if not self.frame.np.transfers:
+            return
+
         folder = self.selected_folder
         realpath = self.frame.np.shares.virtual2real(folder)
 
@@ -837,27 +896,40 @@ class UserBrowse:
         if user is None or user == "":
             return
 
-        self.frame.np.process_request_to_peer(user, slskmessages.UploadQueueNotification(None))
+        self.frame.np.send_message_to_peer(user, slskmessages.UploadQueueNotification(None))
 
         for fn in self.selected_files:
             self.frame.np.transfers.push_file(user, "\\".join([folder, fn]), "\\".join([realpath, fn]), prefix)
             self.frame.np.transfers.check_upload_queue()
 
+    def on_key_press_event(self, widget, event):
+
+        key = Gdk.keyval_name(event.keyval)
+        self.select_files()
+
+        if key in ("C", "c") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
+            files = (widget == self.FileTreeView)
+            self.on_copy_file_path(widget, files)
+        else:
+            # No key match, continue event
+            return False
+
+        widget.stop_emission_by_name("key_press_event")
+        return True
+
     def on_play_files(self, widget, prefix=""):
         start_new_thread(self._on_play_files, (widget, prefix))
 
     def _on_play_files(self, widget, prefix=""):
 
         path = self.frame.np.shares.virtual2real(self.selected_folder)
-        executable = self.frame.np.config.sections["players"]["default"]
-
-        if "$" not in executable:
-            return
 
         for fn in self.selected_files:
-            file = os.sep.join([path, fn])
-            if os.path.exists(file):
-                execute_command(executable, file, background=False)
+            playfile = os.sep.join([path, fn])
+
+            if os.path.exists(playfile):
+                command = self.frame.np.config.sections["players"]["default"]
+                open_file_path(playfile, command)
 
     def find_matches(self):
 
@@ -901,7 +973,7 @@ class UserBrowse:
 
             # Get matching files in the current directory
             resultfiles = []
-            for file in list(self.files.keys()):
+            for file in self.files:
                 if query in file.lower():
                     resultfiles.append(file)
 
@@ -924,18 +996,27 @@ class UserBrowse:
             self.search_position = 0
 
     def on_close(self, widget):
-
         del self.userbrowses.users[self.user]
-        self.frame.np.close_peer_connection(self.conn)
-
         self.userbrowses.remove_page(self.Main)
-        self.Main.destroy()
 
     def on_refresh(self, widget):
+
+        self.refreshing = True
+        self.info_bar.set_visible(False)
+
         self.FolderTreeView.set_sensitive(False)
         self.FileTreeView.set_sensitive(False)
-        self.SaveButton.set_sensitive(False)
-        self.frame.browse_user(self.user)
+
+        self.frame.browse_user(self.user, local_shares_type=self.local_shares_type)
+
+    def on_copy_file_path(self, widget, files=False):
+
+        text = self.selected_folder
+
+        if files and self.selected_files:
+            text = "\\".join([self.selected_folder, self.selected_files[0]])
+
+        self.frame.clip.set_text(text, -1)
 
     def on_copy_url(self, widget):
 
@@ -960,7 +1041,6 @@ class UserBrowse:
             return
 
         path = self.frame.np.shares.virtual2real(self.selected_folder)
-        executable = self.frame.np.config.sections["ui"]["filemanager"]
+        command = self.frame.np.config.sections["ui"]["filemanager"]
 
-        if "$" in executable:
-            execute_command(executable, path)
+        open_file_path(path, command)
