@@ -74,19 +74,20 @@ class UserList:
             str                   # (14) country
         )
 
+        self.column_numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
         self.cols = cols = initialise_columns(
             "buddy_list",
             self.UserListTree,
-            ["status", _("Status"), 25, "pixbuf", None, None],
-            ["country", _("Country"), 25, "pixbuf", None, None],
-            ["user", _("User"), 250, "text", None, None],
-            ["speed", _("Speed"), 150, "number", None, None],
-            ["files", _("Files"), 150, "number", None, None],
-            ["trusted", _("Trusted"), 0, "toggle", None, None],
-            ["notify", _("Notify"), 0, "toggle", None, None],
-            ["privileged", _("Privileged"), 0, "toggle", None, None],
-            ["last_seen", _("Last seen"), 160, "text", None, None],
-            ["comments", _("Comments"), 400, "edit", None, None]
+            ["status", _("Status"), 25, "pixbuf", None],
+            ["country", _("Country"), 25, "pixbuf", None],
+            ["user", _("User"), 250, "text", None],
+            ["speed", _("Speed"), 150, "number", None],
+            ["files", _("Files"), 150, "number", None],
+            ["trusted", _("Trusted"), 0, "toggle", None],
+            ["notify", _("Notify"), 0, "toggle", None],
+            ["privileged", _("Privileged"), 0, "toggle", None],
+            ["last_seen", _("Last seen"), 160, "text", None],
+            ["comments", _("Comments"), 400, "edit", None]
         )
 
         cols["status"].set_sort_column_id(10)
@@ -139,19 +140,19 @@ class UserList:
             row = [
                 self.frame.get_status_image(0),
                 self.frame.get_flag_image(country),
-                username,
+                str(username),
                 "",
                 "",
                 trusted,
                 notify,
                 privileged,
-                last_seen,
-                comment,
+                str(last_seen),
+                str(comment),
                 0,
                 0,
                 0,
                 time_from_epoch,
-                country
+                str(country)
             ]
 
             self.usersmodel.insert(0, row)
@@ -189,7 +190,7 @@ class UserList:
             widget.append_text("")
 
             for user in self.frame.np.config.sections["server"]["userlist"]:
-                widget.append_text(user[0])
+                widget.append_text(str(user[0]))
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
         return show_country_tooltip(widget, x, y, tooltip, 14)
@@ -217,7 +218,7 @@ class UserList:
         iterator = self.usersmodel.get_iter(index)
         value = self.usersmodel.get_value(iterator, pos)
 
-        self.usersmodel.set(iterator, pos, not value)
+        self.usersmodel.set_value(iterator, pos, not value)
 
         self.save_user_list()
 
@@ -240,8 +241,8 @@ class UserList:
 
         for i in self.usersmodel:
             if i[2] == user:
-                self.usersmodel.set(i.iter, 8, last_seen)
-                self.usersmodel.set(i.iter, 13, int(time_from_epoch))
+                self.usersmodel.set_value(i.iter, 8, last_seen)
+                self.usersmodel.set_value(i.iter, 13, int(time_from_epoch))
                 break
 
         if not online:
@@ -255,7 +256,7 @@ class UserList:
 
             for i in self.usersmodel:
                 if i[2] == user:
-                    self.usersmodel.set(iterator, 9, comments)
+                    self.usersmodel.set_value(iterator, 9, comments)
                     break
 
             self.save_user_list()
@@ -263,17 +264,16 @@ class UserList:
     def conn_close(self):
 
         for i in self.usersmodel:
-            self.usersmodel.set(
-                i.iter,
-                0, self.frame.get_status_image(0),
-                3, "",
-                4, "",
-                10, 0,
-                11, 0,
-                12, 0
-            )
+            iterator = i.iter
 
-            if self.usersmodel.get(i.iter, 8)[0] == "":
+            self.usersmodel.set_value(iterator, 0, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)))
+            self.usersmodel.set_value(iterator, 3, "")
+            self.usersmodel.set_value(iterator, 4, "")
+            self.usersmodel.set_value(iterator, 10, 0)
+            self.usersmodel.set_value(iterator, 11, 0)
+            self.usersmodel.set_value(iterator, 12, 0)
+
+            if self.usersmodel.get(iterator, 8)[0] == "":
                 user = i[2]
                 self.set_last_seen(user)
 
@@ -351,8 +351,13 @@ class UserList:
 
     def get_user_status(self, msg):
 
-        user = msg.user
         status = msg.status
+
+        if status < 0:
+            # User doesn't exist, nothing to do
+            return
+
+        user = msg.user
         iterator = self.get_iter(user)
 
         if iterator is None:
@@ -364,16 +369,19 @@ class UserList:
         notify = self.usersmodel.get_value(iterator, 6)
 
         if notify:
-            status_text = [_("User %s is offline"), _("User %s is away"), _("User %s is online")][status]
+            if status == 1:
+                status_text = _("User %s is away")
+            elif status == 2:
+                status_text = _("User %s is online")
+            else:
+                status_text = _("User %s is offline")
+
             log.add(status_text, user)
             self.frame.notifications.new_notification(status_text % user)
 
         img = self.frame.get_status_image(status)
-        self.usersmodel.set(
-            iterator,
-            0, img,
-            10, status
-        )
+        self.usersmodel.set_value(iterator, 0, GObject.Value(GObject.TYPE_OBJECT, img))
+        self.usersmodel.set_value(iterator, 10, GObject.Value(GObject.TYPE_INT64, status))
 
         if status:  # online
             self.set_last_seen(user, online=True)
@@ -388,22 +396,13 @@ class UserList:
         if iterator is None:
             return
 
-        country = msg.country
         hspeed = human_speed(msg.avgspeed)
         hfiles = humanize(msg.files)
 
-        self.usersmodel.set(
-            iterator,
-            3, hspeed,
-            4, hfiles,
-            11, msg.avgspeed,
-            12, msg.files
-        )
-
-        if country is not None and country != "":
-
-            country = "flag_" + country
-            self.set_user_flag(user, country)
+        self.usersmodel.set_value(iterator, 3, hspeed)
+        self.usersmodel.set_value(iterator, 4, hfiles)
+        self.usersmodel.set_value(iterator, 11, GObject.Value(GObject.TYPE_UINT64, msg.avgspeed))
+        self.usersmodel.set_value(iterator, 12, GObject.Value(GObject.TYPE_UINT64, msg.files))
 
     def set_user_flag(self, user, country):
 
@@ -414,22 +413,42 @@ class UserList:
         if user not in (i[2] for i in self.usersmodel):
             return
 
-        self.usersmodel.set(
-            iterator,
-            1, self.frame.get_flag_image(country),
-            14, country
-        )
+        self.usersmodel.set_value(iterator, 1, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)))
+        self.usersmodel.set_value(iterator, 14, "flag_" + country)
 
     def add_to_list(self, user):
 
         if user in (i[2] for i in self.usersmodel):
             return
 
-        row = [self.frame.get_status_image(0), None, user, "", "", False, False, False, _("Never seen"), "", 0, 0, 0, 0, ""]
-        self.usersmodel.append(row)
+        empty_int = 0
+        empty_str = ""
+
+        self.usersmodel.insert_with_valuesv(
+            -1, self.column_numbers,
+            [
+                GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)),
+                GObject.Value(GObject.TYPE_OBJECT, None),
+                user,
+                empty_str,
+                empty_str,
+                False,
+                False,
+                False,
+                _("Never seen"),
+                empty_str,
+                empty_int,
+                empty_int,
+                empty_int,
+                empty_int,
+                empty_str
+            ]
+        )
 
         self.save_user_list()
         self.frame.np.queue.put(slskmessages.AddUser(user))
+
+        # Request user's IP address, so we can get the country
         self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
 
         for widget in self.buddies_combo_entries:
@@ -456,7 +475,7 @@ class UserList:
             for i in self.usersmodel:
                 if i[2] == user:
                     i[9] = comments
-                    self.usersmodel.set(i.iter, 9, comments)
+                    self.usersmodel.set_value(i.iter, 9, comments)
                     break
 
             self.save_user_list()
@@ -499,7 +518,7 @@ class UserList:
 
         for i in self.usersmodel:
             if i[2] == user:
-                self.usersmodel.set(i.iter, 5, widget.get_active())
+                self.usersmodel.set_value(i.iter, 5, widget.get_active())
                 break
 
         self.save_user_list()
@@ -510,7 +529,7 @@ class UserList:
 
         for i in self.usersmodel:
             if i[2] == user:
-                self.usersmodel.set(i.iter, 6, widget.get_active())
+                self.usersmodel.set_value(i.iter, 6, widget.get_active())
                 break
 
         self.save_user_list()
@@ -521,7 +540,7 @@ class UserList:
 
         for i in self.usersmodel:
             if i[2] == user:
-                self.usersmodel.set(i.iter, 7, widget.get_active())
+                self.usersmodel.set_value(i.iter, 7, widget.get_active())
                 break
 
         self.save_user_list()
