@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020-2021 Nicotine+ Team
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -16,17 +16,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import textwrap
 import time
 
-from sys import stdout
-
 from pynicotine.utils import write_log
 
 
-class logger(object):
-    """Coordinates log messages. Has a message history for listeners that are
-    not yet present right at startup."""
+class Logger(object):
 
     def __init__(self):
 
@@ -35,16 +30,31 @@ class logger(object):
         self.folder = ""
         self.file_name = "debug_" + str(int(time.time()))
         self.timestamp_format = "%Y-%m-%d %H:%M:%S"
+        self.log_levels = (0, 1)
 
-    def addwarning(self, msg):
-        """Add a message with the level corresponding to warnings."""
-        self.add(msg, 1)
+    def set_msg_prefix(self, level, msg):
 
-    def adddebug(self, msg):
-        """Add a message with the level corresponding to debug info."""
-        self.add(msg, 6)
+        if level == 1:
+            prefix = "Warn"
+        elif level == 2:
+            prefix = "Search"
+        elif level == 3:
+            prefix = "Conn"
+        elif level == 4:
+            prefix = "Msg"
+        elif level == 5:
+            prefix = "Transfer"
+        elif level == 6:
+            prefix = "Stat"
+        else:
+            prefix = ""
 
-    def add(self, msg, level=0):
+        if prefix:
+            msg = "[%s] %s" % (prefix, msg)
+
+        return msg
+
+    def add(self, msg, msg_args=None, level=0):
         """Add a message. The list of logging levels is as follows:
         None - Deprecated (calls that haven't been updated yet)
         0    - Normal messages and (Human-Readable) Errors
@@ -53,25 +63,48 @@ class logger(object):
         3    - Peer Connections
         4    - Message Contents
         5    - Transfers
-        6    - Connection, Bandwidth and Usage Statistics
+        6    - Statistics
         """
 
-        timestamp = time.localtime()
+        if level not in self.log_levels:
+            return
+
+        msg = self.set_msg_prefix(level, msg)
+
+        if msg_args:
+            msg = msg % msg_args
 
         if self.log_to_file:
             write_log(self.folder, self.file_name, msg, self.timestamp_format)
 
         for callback in self.listeners:
             try:
-                callback(timestamp, level, msg)
+                callback(self.timestamp_format, level, msg)
             except Exception as e:
                 print("Callback on %s failed: %s %s\n%s" % (callback, level, msg, e))
-                pass
 
-    def addlistener(self, callback):
+    def add_warning(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=1)
+
+    def add_search(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=2)
+
+    def add_conn(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=3)
+
+    def add_msg_contents(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=4)
+
+    def add_transfer(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=5)
+
+    def add_debug(self, msg, msg_args=None):
+        self.add(msg, msg_args=msg_args, level=6)
+
+    def add_listener(self, callback):
         self.listeners.add(callback)
 
-    def removelistener(self, callback):
+    def remove_listener(self, callback):
         try:
             self.listeners.remove(callback)
         except KeyError:
@@ -86,52 +119,26 @@ class logger(object):
     def set_timestamp_format(self, timestamp_format):
         self.timestamp_format = timestamp_format
 
-
-useconsole = True
-try:
-    CONSOLEENCODING = stdout.encoding
-except AttributeError:
-    print("stdout does not have an encoding attribute - disabling console logging.")
-    useconsole = False
-
-if useconsole:
-    if not CONSOLEENCODING or CONSOLEENCODING.lower() == 'ascii':
-        # ASCII is quite improbable, lets just hope the user hasnt set up
-        # everything properly and its really UTF8
-        CONSOLEENCODING = 'UTF8'
-    CONSOLEWIDTH = 80
-    try:
-        # Fixed, you better not resize your window!
-        import sys
-        import fcntl
-        import termios
-        import struct
-        data = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, '1234')
-        CONSOLEWIDTH = struct.unpack('hh', data)[1]
-    except Exception:
-        pass
-
-    TIMEFORMAT = "%a %H:%M "
-
-    wrapper = textwrap.TextWrapper()
-    wrapper.width = CONSOLEWIDTH
-    wrapper.subsequent_indent = " " * len(time.strftime(TIMEFORMAT))
-    wrapper.expand_tabs = False
-    wrapper.replace_whitespace = True
+    def set_log_levels(self, levels):
+        self.log_levels = levels
 
 
-def consolelogger(timestamp, level, msg):
-    if level in (1,):
-        wrapper.initial_indent = time.strftime(TIMEFORMAT, timestamp)
-        for i in wrapper.wrap(msg):
-            print(i)
-    else:
-        pass
+class Console(object):
+
+    def __init__(self, logger):
+        self.log_levels = (1,)
+        logger.add_listener(self.console_logger)
+
+    def console_logger(self, timestamp_format, level, msg):
+        if level in self.log_levels:
+            print("[" + time.strftime(timestamp_format) + "] " + msg)
+
+    def set_log_levels(self, levels):
+        self.log_levels = levels
 
 
 try:
     log
 except NameError:
-    log = logger()
-    if useconsole:
-        log.addlistener(consolelogger)  # by default let's display important stuff in the console
+    log = Logger()
+    console = Console(log)
