@@ -103,7 +103,7 @@ class PluginHandler(object):
             from importlib.machinery import SourceFileLoader
             plugin = SourceFileLoader(pluginname, os.path.join(path, '__init__.py')).load_module()
 
-        instance = plugin.Plugin(self)
+        instance = plugin.Plugin(self, enable_plugin=False)
         self.plugin_settings(pluginname, instance)
         instance.LoadNotification()
 
@@ -141,10 +141,10 @@ class PluginHandler(object):
     def list_installed_plugins(self):
         pluginlist = []
 
-        for dir in self.plugindirs:
-            if os.path.exists(dir):
-                for file in os.listdir(dir):
-                    if file not in pluginlist and os.path.isdir(os.path.join(dir, file)):
+        for folder in self.plugindirs:
+            if os.path.isdir(folder):
+                for file in os.listdir(folder):
+                    if file not in pluginlist and os.path.isdir(os.path.join(folder, file)):
                         pluginlist.append(file)
 
         return pluginlist
@@ -235,23 +235,27 @@ class PluginHandler(object):
         return self._trigger_command(command, user, args, public_command=False)
 
     def _trigger_command(self, command, source, args, public_command):
+
         for module, plugin in self.enabled_plugins.items():
             try:
                 if plugin.PLUGIN is None:
                     continue
 
                 if public_command:
-                    ret = plugin.PLUGIN.PublicCommandEvent(command, source, args)
+                    return_value = plugin.PLUGIN.PublicCommandEvent(command, source, args)
                 else:
-                    ret = plugin.PLUGIN.PrivateCommandEvent(command, source, args)
+                    return_value = plugin.PLUGIN.PrivateCommandEvent(command, source, args)
 
-                if ret is not None:
-                    if ret == returncode['zap']:
-                        return True
-                    elif ret == returncode['pass']:
-                        pass
-                    else:
-                        log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': str(ret)})
+                if return_value is None:
+                    # Nothing changed, continue to the next plugin
+                    continue
+
+                if return_value == returncode['zap']:
+                    return True
+                elif return_value == returncode['pass']:
+                    pass
+                else:
+                    log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': str(return_value)})
 
             except Exception:
                 from traceback import extract_stack
@@ -269,28 +273,31 @@ class PluginHandler(object):
         return False
 
     def trigger_event(self, function, args):
-        """Triggers an event for the plugins. Since events and notifications
+        """ Triggers an event for the plugins. Since events and notifications
         are precisely the same except for how n+ responds to them, both can be
-        triggered by this function."""
-
-        hotpotato = args
+        triggered by this function. """
 
         for module, plugin in self.enabled_plugins.items():
             try:
-                ret = getattr(plugin.PLUGIN, function)(*hotpotato)
+                return_value = getattr(plugin.PLUGIN, function)(*args)
 
-                if ret is not None and not isinstance(ret, tuple):
-                    if ret == returncode['zap']:
-                        return None
-                    elif ret == returncode['break']:
-                        return hotpotato
-                    elif ret == returncode['pass']:
-                        pass
-                    else:
-                        log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': ret})
+                if return_value is None:
+                    # Nothing changed, continue to the next plugin
+                    continue
 
-                if ret is not None:
-                    hotpotato = ret
+                if isinstance(return_value, tuple):
+                    # The original args were modified, update them
+                    args = return_value
+                    continue
+
+                if return_value == returncode['zap']:
+                    return None
+                elif return_value == returncode['break']:
+                    return args
+                elif return_value == returncode['pass']:
+                    pass
+                else:
+                    log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': return_value})
 
             except Exception:
                 from traceback import extract_stack
@@ -305,7 +312,7 @@ class PluginHandler(object):
                     'area': ''.join(format_list(extract_tb(sys.exc_info()[2])))
                 })
 
-        return hotpotato
+        return args
 
     def search_request_notification(self, searchterm, user, searchid):
         self.trigger_event("SearchRequestNotification", (searchterm, user, searchid))
@@ -378,12 +385,21 @@ class PluginHandler(object):
     def leave_chatroom_notification(self, room):
         self.trigger_event("LeaveChatroomNotification", (room,))
 
+    def user_join_chatroom_notification(self, room, user):
+        self.trigger_event("UserJoinChatroomNotification", (room, user,))
+
+    def user_leave_chatroom_notification(self, room, user):
+        self.trigger_event("UserLeaveChatroomNotification", (room, user,))
+
     def upload_queued_notification(self, user, virtualfile, realfile):
         self.trigger_event("UploadQueuedNotification", (user, virtualfile, realfile))
 
     def user_stats_notification(self, user, stats):
         self.trigger_event("UserStatsNotification", (user, stats))
 
+    def shutdown_notification(self):
+        self.trigger_event("ShutdownNotification", (),)
+
     """ Other Functions """
 
     def log(self, text):
@@ -494,11 +510,16 @@ class BasePlugin(object):
     __publiccommands__ = []
     __privatecommands__ = []
 
-    def __init__(self, parent):
+    def __init__(self, parent, enable_plugin=True):
+
         # Never override this function, override init() instead
         self.parent = parent
         self.frame = parent.frame
 
+        if not enable_plugin:
+            # Plugin was loaded, but not enabled yet
+            return
+
         self.init()
         for (trigger, func) in self.__publiccommands__:
             self.frame.chatrooms.CMDS.add('/' + trigger + ' ')
@@ -575,12 +596,21 @@ class BasePlugin(object):
     def LeaveChatroomNotification(self, room):  # noqa
         pass
 
+    def UserJoinChatroomNotification(self, room, user):  # noqa
+        pass
+
+    def UserLeaveChatroomNotification(self, room, user):  # noqa
+        pass
+
     def UploadQueuedNotification(self, user, virtualfile, realfile):  # noqa
         pass
 
     def UserStatsNotification(self, user, stats):  # noqa
         pass
 
+    def ShutdownNotification(self):  # noqa
+        pass
+
     # The following are functions to make your life easier,
     # you shouldn't override them.
     def log(self, text):
