@@ -49,6 +49,15 @@ from pynicotine.utils import clean_file
 from pynicotine.utils import unescape
 
 
+class UserAddr:
+
+    __slots__ = "addr", "status"
+
+    def __init__(self, addr=None, status=None):
+        self.addr = addr
+        self.status = status
+
+
 class PeerConnection:
     """
     Holds information about a peer connection. Not every field may be set
@@ -314,6 +323,8 @@ class NetworkEventProcessor:
                         i.conntimer.cancel()
 
                     found_conn = True
+
+                    # Deliver our messages to the peer
                     self.process_conn_messages(i, conn)
                     break
 
@@ -354,8 +365,8 @@ class NetworkEventProcessor:
         if conn is not None and conn.conn is not None:
             """ We have initiated a connection previously, and it's ready """
 
-            message.conn = conn.conn
-            self.queue.put(message)
+            conn.msgs.append(message)
+            self.process_conn_messages(conn, conn.conn)
 
         elif conn is not None:
             """ Connection exists but is not ready yet, add new messages to it """
@@ -596,7 +607,7 @@ class NetworkEventProcessor:
         if cc == "-":
             cc = ""
 
-        self.ui_callback.has_user_flag(user, "flag_" + cc)
+        self.ui_callback.has_user_flag(user, cc)
 
         # From this point on all paths should call
         # self.pluginhandler.user_resolve_notification precisely once
@@ -624,7 +635,8 @@ class NetworkEventProcessor:
 
     def process_conn_messages(self, peerconn, conn):
 
-        """ We may have messages from our peer, represent these in our UI """
+        """ A connection is established with the peer, time to queue up our peer
+        messages for delivery """
 
         for j in peerconn.msgs:
 
@@ -650,8 +662,10 @@ class NetworkEventProcessor:
 
     def out_conn(self, msg):
 
-        """ Direct connection to peer was successful, send pierce firewall message
-        and process any messages associated with the connection """
+        """ Networking thread told us that the connection to the peer was successful.
+        If we connected directly to the peer, send a PeerInit message. If we connected
+        as a result of an indirect connect request by the peer, send a PierceFirewall
+        message. Queue up any messages we want to deliver to the peer. """
 
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
@@ -675,7 +689,7 @@ class NetworkEventProcessor:
                     'messages': i.msgs
                 })
 
-                # Update UI with contents from messages
+                # Deliver our messages to the peer
                 self.process_conn_messages(i, conn)
 
                 break
@@ -707,7 +721,7 @@ class NetworkEventProcessor:
                     'messages': i.msgs
                 })
 
-                # Update UI with contents from messages
+                # Deliver our messages to the peer
                 self.process_conn_messages(i, conn)
 
                 break
@@ -1029,6 +1043,8 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if msg.success:
+            self.queue.put(slskmessages.SetStatus((not self.ui_callback.away) + 1))
+            self.queue.put(slskmessages.AddUser(self.config.sections["server"]["login"]))
 
             self.transfers = transfers.Transfers(self.peerconns, self.queue, self, self.users,
                                                  self.network_callback, self.ui_callback.notifications, self.pluginhandler)
@@ -1037,11 +1053,15 @@ class NetworkEventProcessor:
             if msg.ip is not None:
                 self.ipaddress = msg.ip
 
+            for i in self.config.sections["server"]["userlist"]:
+                user = i[0]
+                if user not in self.watchedusers:
+                    self.queue.put(slskmessages.AddUser(user))
+
             self.privatechat, self.chatrooms, self.userinfo, self.userbrowse, self.search, downloads, uploads, self.userlist, self.interests = self.ui_callback.init_interface(msg)
 
             self.transfers.set_transfer_views(downloads, uploads)
             self.shares.send_num_shared_folders_files()
-            self.queue.put(slskmessages.SetStatus((not self.ui_callback.away) + 1))
 
             for thing in self.config.sections["interests"]["likes"]:
                 if thing and isinstance(thing, str):
@@ -1068,6 +1088,7 @@ class NetworkEventProcessor:
             self.queue.put(slskmessages.RoomList())
 
             self.queue.put(slskmessages.PrivateRoomToggle(self.config.sections["server"]["private_chatrooms"]))
+            self.pluginhandler.server_connect_notification()
         else:
             self.manualdisconnect = True
             self.set_status(_("Can not log in, reason: %s"), (msg.reason))
@@ -1135,14 +1156,7 @@ class NetworkEventProcessor:
         log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.privatechat is not None:
-            event = self.pluginhandler.incoming_private_chat_event(msg.user, msg.msg)
-
-            if event is not None:
-                (u, msg.msg) = event
-                self.privatechat.show_message(msg, msg.msg, msg.newmessage)
-
-                self.pluginhandler.incoming_private_chat_notification(msg.user, msg.msg)
-
+            self.privatechat.show_message(msg, msg.msg, msg.newmessage)
             self.queue.put(slskmessages.MessageAcked(msg.msgid))
 
     def user_joined_room(self, msg):
@@ -1328,17 +1342,14 @@ class NetworkEventProcessor:
 
         self.watchedusers.add(msg.user)
 
-        if not msg.userexists:
-            if msg.user not in self.users:
-                self.users[msg.user] = UserAddr(status=-1)
-
-        if msg.status is not None:
-            self.get_user_status(msg)
-        elif msg.userexists and msg.status is None:
+        if msg.userexists and msg.status is None:
+            # Legacy support (Soulfind server)
             self.queue.put(slskmessages.GetUserStatus(msg.user))
+        else:
+            self.get_user_status(msg, log_contents=False)
 
         if msg.files is not None:
-            self.get_user_stats(msg)
+            self.get_user_stats(msg, log_contents=False)
 
     def privileged_users(self, msg):
 
@@ -1347,8 +1358,6 @@ class NetworkEventProcessor:
         if self.transfers is not None:
             self.transfers.set_privileged_users(msg.users)
             log.add(_("%i privileged users"), (len(msg.users)))
-            self.queue.put(slskmessages.AddUser(self.config.sections["server"]["login"]))
-            self.pluginhandler.server_connect_notification()
 
     def add_to_privileged(self, msg):
 
@@ -1411,14 +1420,17 @@ class NetworkEventProcessor:
         if self.search is not None:
             self.search.wish_list.set_interval(msg)
 
-    def get_user_status(self, msg):
+    def get_user_status(self, msg, log_contents=True):
 
-        log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
+        if log_contents:
+            log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
+
+        if msg.status is None:
+            msg.status = -1
 
-        # Causes recursive requests when privileged?
-        # self.queue.put(slskmessages.AddUser(msg.user))
         if msg.user in self.users:
-            if msg.status == 0:
+            if msg.status <= 0:
+                # User went offline, reset stored IP address
                 self.users[msg.user] = UserAddr(status=msg.status)
             else:
                 self.users[msg.user].status = msg.status
@@ -1429,8 +1441,6 @@ class NetworkEventProcessor:
             if msg.privileged == 1:
                 if self.transfers is not None:
                     self.transfers.add_to_privileged(msg.user)
-                else:
-                    log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if self.interests is not None:
             self.interests.get_user_status(msg)
@@ -1460,9 +1470,10 @@ class NetworkEventProcessor:
         if self.userinfo is not None:
             self.userinfo.show_interests(msg)
 
-    def get_user_stats(self, msg):
+    def get_user_stats(self, msg, log_contents=True):
 
-        log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
+        if log_contents:
+            log.add_msg_contents("%s %s", (msg.__class__, self.contents(msg)))
 
         if msg.user == self.config.sections["server"]["login"]:
             self.speed = msg.avgspeed
@@ -1572,7 +1583,7 @@ class NetworkEventProcessor:
                 # probably impossible to do this
                 if i.username != self.config.sections["server"]["login"]:
                     indeterminate_progress = change_page = False
-                    self.userinfo.show_user(i.username, msg.conn, msg, indeterminate_progress, change_page)
+                    self.userinfo.show_user(i.username, None, msg, indeterminate_progress, change_page)
                     break
 
     def user_info_request(self, msg):
@@ -1672,7 +1683,7 @@ class NetworkEventProcessor:
             if i.conn is conn and self.userbrowse is not None:
                 if i.username != self.config.sections["server"]["login"]:
                     indeterminate_progress = change_page = False
-                    self.userbrowse.show_user(i.username, msg.conn, msg, indeterminate_progress, change_page)
+                    self.userbrowse.show_user(i.username, None, msg, indeterminate_progress, change_page)
                     break
 
     def file_search_result(self, msg):
@@ -2157,10 +2168,3 @@ class NetworkEventProcessor:
     def ignore(self, msg):
         # Ignore received message
         pass
-
-
-class UserAddr:
-
-    def __init__(self, addr=None, status=None):
-        self.addr = addr
-        self.status = status
