@@ -1,4 +1,4 @@
-# Copyright (C) 2020 Nicotine+ Team
+# Copyright (C) 2020-2021 Nicotine+ Team
 # Copyright (C) 2007 daelstorm. All rights reserved.
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 #
@@ -30,7 +30,6 @@ import threading
 import time
 
 from errno import EINTR
-from gettext import gettext as _
 from itertools import islice
 from random import uniform
 
@@ -45,6 +44,7 @@ from pynicotine.slskmessages import AdminMessage
 from pynicotine.slskmessages import BranchLevel
 from pynicotine.slskmessages import BranchRoot
 from pynicotine.slskmessages import CantConnectToPeer
+from pynicotine.slskmessages import CantCreateRoom
 from pynicotine.slskmessages import ChangePassword
 from pynicotine.slskmessages import CheckPrivileges
 from pynicotine.slskmessages import ChildDepth
@@ -124,6 +124,7 @@ from pynicotine.slskmessages import QueuedDownloads
 from pynicotine.slskmessages import QueueFailed
 from pynicotine.slskmessages import QueueUpload
 from pynicotine.slskmessages import Recommendations
+from pynicotine.slskmessages import RelatedSearch
 from pynicotine.slskmessages import Relogged
 from pynicotine.slskmessages import RemoveThingIHate
 from pynicotine.slskmessages import RemoveThingILike
@@ -140,14 +141,14 @@ from pynicotine.slskmessages import SayChatroom
 from pynicotine.slskmessages import SearchInactivityTimeout
 from pynicotine.slskmessages import SearchParent
 from pynicotine.slskmessages import SearchRequest
-from pynicotine.slskmessages import SendSpeed
+from pynicotine.slskmessages import SendConnectToken
+from pynicotine.slskmessages import SendDownloadSpeed
 from pynicotine.slskmessages import SendUploadSpeed
 from pynicotine.slskmessages import ServerConn
 from pynicotine.slskmessages import ServerMessage
 from pynicotine.slskmessages import ServerPing
 from pynicotine.slskmessages import SetCurrentConnectionCount
 from pynicotine.slskmessages import SetDownloadLimit
-from pynicotine.slskmessages import SetGeoBlock
 from pynicotine.slskmessages import SetStatus
 from pynicotine.slskmessages import SetUploadLimit
 from pynicotine.slskmessages import SetWaitPort
@@ -198,7 +199,7 @@ else:
     try:
         resource.setrlimit(resource.RLIMIT_NOFILE, (maxfilelimit, maxfilelimit))
     except Exception as e:
-        log.add_warning(_("Failed to set RLIMIT_NOFILE: %s") % e)
+        log.add_warning("Failed to set RLIMIT_NOFILE: %s" % e)
 
     """ Set the maximum number of open sockets to a lower value than the hard limit,
     otherwise we just waste resources.
@@ -288,11 +289,12 @@ class SlskProtoThread(threading.Thread):
         MessageAcked: 23,
         FileSearch: 26,
         SetStatus: 28,
-        ServerPing: 32,  # Depreciated
-        SendSpeed: 34,  # Depreciated
+        ServerPing: 32,               # Deprecated
+        SendConnectToken: 33,         # Deprecated
+        SendDownloadSpeed: 34,        # Deprecated
         SharedFoldersFiles: 35,
         GetUserStats: 36,
-        QueuedDownloads: 40,  # Depreciated
+        QueuedDownloads: 40,          # Deprecated
         Relogged: 41,
         UserSearch: 42,
         AddThingILike: 51,
@@ -300,23 +302,23 @@ class SlskProtoThread(threading.Thread):
         Recommendations: 54,
         GlobalRecommendations: 56,
         UserInterests: 57,
-        PlaceInLineResponse: 60,  # Depreciated
-        RoomAdded: 62,  # Depreciated
-        RoomRemoved: 63,  # Depreciated
+        PlaceInLineResponse: 60,      # Deprecated
+        RoomAdded: 62,                # Deprecated
+        RoomRemoved: 63,              # Deprecated
         RoomList: 64,
-        ExactFileSearch: 65,  # Depreciated
+        ExactFileSearch: 65,          # Deprecated
         AdminMessage: 66,
-        GlobalUserList: 67,  # Depreciated
-        TunneledMessage: 68,  # Depreciated
+        GlobalUserList: 67,           # Deprecated
+        TunneledMessage: 68,          # Deprecated
         PrivilegedUsers: 69,
         HaveNoParent: 71,
         SearchParent: 73,
-        ParentMinSpeed: 83,  # Unused
-        ParentSpeedRatio: 84,  # Unused
-        ParentInactivityTimeout: 86,  # Depreciated
-        SearchInactivityTimeout: 87,  # Depreciated
-        MinParentsInCache: 88,  # Depreciated
-        DistribAliveInterval: 90,  # Depreciated
+        ParentMinSpeed: 83,           # Unused
+        ParentSpeedRatio: 84,         # Unused
+        ParentInactivityTimeout: 86,  # Deprecated
+        SearchInactivityTimeout: 87,  # Deprecated
+        MinParentsInCache: 88,        # Deprecated
+        DistribAliveInterval: 90,     # Deprecated
         AddToPrivileged: 91,
         CheckPrivileges: 92,
         SearchRequest: 93,
@@ -339,9 +341,9 @@ class SlskProtoThread(threading.Thread):
         GivePrivileges: 123,
         NotifyPrivileges: 124,
         AckNotifyPrivileges: 125,
-        BranchLevel: 126,  # Unimplemented
-        BranchRoot: 127,  # Unimplemented
-        ChildDepth: 129,  # Unimplemented
+        BranchLevel: 126,             # Unimplemented
+        BranchRoot: 127,              # Unimplemented
+        ChildDepth: 129,              # Unimplemented
         PrivateRoomUsers: 133,
         PrivateRoomAddUser: 134,
         PrivateRoomRemoveUser: 135,
@@ -360,7 +362,9 @@ class SlskProtoThread(threading.Thread):
         JoinPublicRoom: 150,
         LeavePublicRoom: 151,
         PublicRoomMessage: 152,
-        CantConnectToPeer: 1001
+        RelatedSearch: 153,           # Deprecated ?
+        CantConnectToPeer: 1001,      # Deprecated
+        CantCreateRoom: 1003
     }
 
     peercodes = {
@@ -375,7 +379,7 @@ class SlskProtoThread(threading.Thread):
         FolderContentsResponse: 37,
         TransferRequest: 40,
         TransferResponse: 41,
-        PlaceholdUpload: 42,  # Depreciated
+        PlaceholdUpload: 42,          # Deprecated
         QueueUpload: 43,
         PlaceInQueue: 44,
         UploadFailed: 46,
@@ -388,13 +392,13 @@ class SlskProtoThread(threading.Thread):
     distribclasses = {
         0: DistribAlive,
         3: DistribSearch,
-        4: DistribBranchLevel,  # Unimplemented
-        5: DistribBranchRoot,  # Unimplemented
-        7: DistribChildDepth,  # Unimplemented
+        4: DistribBranchLevel,        # Unimplemented
+        5: DistribBranchRoot,         # Unimplemented
+        7: DistribChildDepth,         # Unimplemented
         93: DistribServerSearch
     }
 
-    IN_PROGRESS_STALE_AFTER = 5
+    IN_PROGRESS_STALE_AFTER = 2
     CONNECTION_MAX_IDLE = 60
     CONNCOUNT_UI_INTERVAL = 0.5
 
@@ -405,9 +409,12 @@ class SlskProtoThread(threading.Thread):
         """
         threading.Thread.__init__(self)
 
+        self.name = "NetworkThread"
+
         self._ui_callback = ui_callback
         self._queue = queue
         self._want_abort = False
+        self._server_disconnect = True
         self._bindip = bindip
         self._config = config
         self._eventprocessor = eventprocessor
@@ -426,7 +433,7 @@ class SlskProtoThread(threading.Thread):
         self._conns = {}
         self._connsinprogress = {}
         self._uploadlimit = (self._calc_upload_limit_none, 0)
-        self._downloadlimit = (self._calc_download_limit_by_total, self._config.sections["transfers"]["downloadlimit"])
+        self._downloadlimit = (self._calc_download_limit_by_total, 0)
         self._ulimits = {}
         self._dlimits = {}
         self.total_uploads = 0
@@ -434,11 +441,6 @@ class SlskProtoThread(threading.Thread):
 
         self.last_conncount_ui_update = time.time()
 
-        # GeoIP Config
-        self._geoip = None
-        # GeoIP Database
-        self.geoip = self._eventprocessor.geoip
-
         portrange = (port, port) if port else config.sections["server"]["portrange"]
         listenport = None
 
@@ -620,13 +622,13 @@ class SlskProtoThread(threading.Thread):
                 msgs.append(msg)
 
             else:
-                msgs.append(_("Server message type %(type)i size %(size)i contents %(msg_buffer)s unknown") % {'type': msgtype, 'size': msgsize - 4, 'msg_buffer': msg_buffer[8:msgsize + 4].__repr__()})
+                msgs.append("Server message type %(type)i size %(size)i contents %(msg_buffer)s unknown" % {'type': msgtype, 'size': msgsize - 4, 'msg_buffer': msg_buffer[8:msgsize + 4].__repr__()})
 
             msg_buffer = msg_buffer[msgsize + 4:]
 
         return msgs, msg_buffer
 
-    def process_file_input(self, conn, msg_buffer):
+    def process_file_input(self, conn, msg_buffer, conns):
         """ We have a "F" connection (filetransfer), peer has sent us
         something, this function retrieves messages
         from the msg_buffer, creates message objects and returns them
@@ -650,6 +652,8 @@ class SlskProtoThread(threading.Thread):
                     conn.filedown.file.write(addedbytes)
                 except IOError as strerror:
                     self._ui_callback([FileError(conn, conn.filedown.file, strerror)])
+                    self._ui_callback([ConnClose(conn.conn, conn.addr)])
+                    self.close_connection(conns, conn.conn)
                 except ValueError:
                     pass
 
@@ -659,10 +663,10 @@ class SlskProtoThread(threading.Thread):
             """ Depending on the number of active downloads, the cooldown for UI callbacks
             can be up to 15 seconds per transfer. We use a bit of randomness to give the
             illusion that downloads are updated often. """
+            finished = ((leftbytes - addedbyteslen) == 0)
             cooldown = max(1.0, min(self.total_downloads * uniform(0.8, 1.0), 15))
 
-            if (leftbytes - addedbyteslen) == 0 or \
-                    (curtime - conn.lastcallback) > cooldown:
+            if finished or (curtime - conn.lastcallback) > cooldown:
 
                 """ We save resources by not sending data back to the UI every time
                 a part of a file is downloaded """
@@ -670,6 +674,10 @@ class SlskProtoThread(threading.Thread):
                 self._ui_callback([DownloadFile(conn.conn, addedbyteslen, conn.filedown.file)])
                 conn.lastcallback = curtime
 
+            if finished:
+                self._ui_callback([ConnClose(conn.conn, conn.addr)])
+                self.close_connection(conns, conn.conn)
+
             conn.filereadbytes += addedbyteslen
             msg_buffer = msg_buffer[leftbytes:]
 
@@ -682,6 +690,8 @@ class SlskProtoThread(threading.Thread):
                         conn.fileupl.file.seek(offset)
                     except IOError as strerror:
                         self._ui_callback([FileError(conn, conn.fileupl.file, strerror)])
+                        self._ui_callback([ConnClose(conn.conn, conn.addr)])
+                        self.close_connection(conns, conn.conn)
                     except ValueError:
                         pass
 
@@ -760,7 +770,7 @@ class SlskProtoThread(threading.Thread):
                             msg.parse_network_message(msg_buffer[8:msgsize + 4])
 
                         except Exception as error:
-                            host = port = _("unknown")
+                            host = port = "unknown"
                             msgname = str(self.peerclasses[msgtype]).split(".")[-1]
                             print("Error parsing %s:" % msgname, error)
 
@@ -772,7 +782,7 @@ class SlskProtoThread(threading.Thread):
                                 if conn.addr is not None:
                                     host = conn.addr[0]
                                     port = conn.addr[1]
-                            debugmessage = _("There was an error while unpacking Peer message type %(type)s size %(size)i contents %(msg_buffer)s from user: %(user)s, %(host)s:%(port)s") % {'type': msgname, 'size': msgsize - 4, 'msg_buffer': msg_buffer[8:msgsize + 4].__repr__(), 'user': conn.init.user, 'host': host, 'port': port}
+                            debugmessage = "There was an error while unpacking Peer message type %(type)s size %(size)i contents %(msg_buffer)s from user: %(user)s, %(host)s:%(port)s" % {'type': msgname, 'size': msgsize - 4, 'msg_buffer': msg_buffer[8:msgsize + 4].__repr__(), 'user': conn.init.user, 'host': host, 'port': port}
                             msgs.append(debugmessage)
 
                             del msg
@@ -785,7 +795,7 @@ class SlskProtoThread(threading.Thread):
                         msgs.append(debugmessage)
 
                 else:
-                    host = port = _("unknown")
+                    host = port = "unknown"
 
                     if conn.init.conn is not None and conn.addr is not None:
                         host = conn.addr[0]
@@ -803,12 +813,12 @@ class SlskProtoThread(threading.Thread):
                         newbuf += char
                         x += 1
 
-                    debugmessage = _("Peer message type %(type)s size %(size)i contents %(msg_buffer)s unknown, from user: %(user)s, %(host)s:%(port)s") % {'type': msgtype, 'size': msgsize - 4, 'msg_buffer': newbuf, 'user': conn.init.user, 'host': host, 'port': port}
+                    debugmessage = "Peer message type %(type)s size %(size)i contents %(msg_buffer)s unknown, from user: %(user)s, %(host)s:%(port)s" % {'type': msgtype, 'size': msgsize - 4, 'msg_buffer': newbuf, 'user': conn.init.user, 'host': host, 'port': port}
                     msgs.append(debugmessage)
 
             else:
                 # Unknown Message type
-                msgs.append(_("Can't handle connection type %s") % (conn.init.type))
+                msgs.append("Can't handle connection type %s" % (conn.init.type))
 
             if msgsize >= 0:
                 msg_buffer = msg_buffer[msgsize + 4:]
@@ -840,7 +850,7 @@ class SlskProtoThread(threading.Thread):
                 msgs.append(msg)
 
             else:
-                msgs.append(_("Distrib message type %(type)i size %(size)i contents %(msg_buffer)s unknown") % {'type': msgtype, 'size': msgsize - 1, 'msg_buffer': msg_buffer[5:msgsize + 4].__repr__()})
+                msgs.append("Distrib message type %(type)i size %(size)i contents %(msg_buffer)s unknown" % {'type': msgtype, 'size': msgsize - 1, 'msg_buffer': msg_buffer[5:msgsize + 4].__repr__()})
                 self._ui_callback([ConnClose(conn.conn, conn.addr)])
                 conn.conn.close()
                 conn.conn = None
@@ -905,8 +915,13 @@ class SlskProtoThread(threading.Thread):
             )
 
     def close_connection(self, connection_list, connection):
-        connection.close()
-        del connection_list[connection]
+        try:
+            connection.close()
+            del connection_list[connection]
+
+        except KeyError:
+            # Already removed
+            pass
 
     def process_queue(self, queue, conns, connsinprogress, server_socket, maxsockets=MAXSOCKETS):
         """ Processes messages sent by UI thread. server_socket is a server connection
@@ -934,8 +949,8 @@ class SlskProtoThread(threading.Thread):
                         needsleep = True
 
                 except Exception as error:
-                    print(_("Error packaging message: %(type)s %(msg_obj)s, %(error)s") % {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj), 'error': str(error)})
-                    self._ui_callback([_("Error packaging message: %(type)s %(msg_obj)s, %(error)s") % {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj), 'error': str(error)}])
+                    print("Error packaging message: %(type)s %(msg_obj)s, %(error)s" % {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj), 'error': str(error)})
+                    self._ui_callback(["Error packaging message: %(type)s %(msg_obj)s, %(error)s" % {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj), 'error': str(error)}])
 
             elif issubclass(msg_obj.__class__, PeerMessage):
                 if msg_obj.conn in conns:
@@ -968,22 +983,13 @@ class SlskProtoThread(threading.Thread):
                         self._ui_callback([msg_obj])
 
                     else:
-                        checkuser = 1
-
-                        if msg_obj.__class__ is FileSearchResult and msg_obj.geoip and self.geoip and self._geoip:
-                            cc = self.geoip.get_all(conns[msg_obj.conn].addr[0]).country_short
-
-                            if (cc == "-" and self._geoip[0]) or (cc != "-" and self._geoip[1][0].find(cc) >= 0):
-                                checkuser = 0
-
-                        if checkuser:
-                            msg = msg_obj.make_network_message()
-                            conns[msg_obj.conn].obuf.extend(struct.pack("<ii", len(msg) + 4, self.peercodes[msg_obj.__class__]))
-                            conns[msg_obj.conn].obuf.extend(msg)
+                        msg = msg_obj.make_network_message()
+                        conns[msg_obj.conn].obuf.extend(struct.pack("<ii", len(msg) + 4, self.peercodes[msg_obj.__class__]))
+                        conns[msg_obj.conn].obuf.extend(msg)
 
                 else:
                     if msg_obj.__class__ not in [PeerInit, PierceFireWall, FileSearchResult]:
-                        log.add_conn(_("Can't send the message over the closed connection: %(type)s %(msg_obj)s"), {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj)})
+                        log.add_conn("Can't send the message over the closed connection: %(type)s %(msg_obj)s", {'type': msg_obj.__class__, 'msg_obj': vars(msg_obj)})
 
             elif issubclass(msg_obj.__class__, InternalMessage):
                 if msg_obj.__class__ is ServerConn:
@@ -1011,12 +1017,20 @@ class SlskProtoThread(threading.Thread):
                             server_socket.close()
 
                 elif msg_obj.__class__ is ConnClose and msg_obj.conn in conns:
-                    self._ui_callback([ConnClose(msg_obj.conn, conns[msg_obj.conn].addr)])
-                    self.close_connection(conns, msg_obj.conn)
+                    conn = msg_obj.conn
+
+                    if msg_obj.callback:
+                        self._ui_callback([ConnClose(conn, conns[conn].addr)])
+
+                    if conn == server_socket:
+                        # Disconnected from server, clean up connections and queue
+                        self.server_disconnect()
+
+                    self.close_connection(conns, conn)
 
                 elif msg_obj.__class__ is OutConn:
                     if msg_obj.addr[1] == 0:
-                        self._ui_callback([ConnectError(msg_obj, (0, "Port cannot be zero"))])
+                        self._ui_callback([ConnectError(msg_obj, "Port cannot be zero")])
 
                     elif maxsockets == -1 or numsockets < maxsockets:
                         try:
@@ -1038,7 +1052,7 @@ class SlskProtoThread(threading.Thread):
                             self._ui_callback([ConnectError(msg_obj, err)])
                             conn.close()
                     else:
-                        self._ui_callback([ConnectError(msg_obj)])
+                        self._ui_callback([ConnectError(msg_obj, "Connection limit reached")])
 
                 elif msg_obj.__class__ is DownloadFile and msg_obj.conn in conns:
                     conns[msg_obj.conn].filedown = msg_obj
@@ -1054,9 +1068,6 @@ class SlskProtoThread(threading.Thread):
                     conns[msg_obj.conn].fileupl = msg_obj
                     self._reset_counters(conns)
 
-                elif msg_obj.__class__ is SetGeoBlock:
-                    self._geoip = msg_obj.config
-
                 elif msg_obj.__class__ is SetUploadLimit:
                     if msg_obj.uselimit:
                         if msg_obj.limitby:
@@ -1117,6 +1128,8 @@ class SlskProtoThread(threading.Thread):
 
                 except IOError as strerror:
                     self._ui_callback([FileError(conn, conn.fileupl.file, strerror)])
+                    self._ui_callback([ConnClose(i, conn.addr)])
+                    self.close_connection(conns, i)
 
                 except ValueError:
                     pass
@@ -1166,7 +1179,7 @@ class SlskProtoThread(threading.Thread):
             conn.lastreadlength = limit
             conn.readbytes2 += len(data)
 
-        if not data:
+        if not data and not conn.obuf:  # Make sure we don't have data to send on this connection
             self._ui_callback([ConnClose(i, conn.addr)])
             self.close_connection(conns, i)
 
@@ -1185,6 +1198,11 @@ class SlskProtoThread(threading.Thread):
 
         while not self._want_abort:
 
+            if self._server_disconnect:
+                # We're not connected to the server at the moment
+                time.sleep(0.2)
+                continue
+
             if not queue.empty():
                 conns, connsinprogress, server_socket = self.process_queue(queue, conns, connsinprogress, server_socket)
                 self._server_socket = server_socket
@@ -1238,7 +1256,7 @@ class SlskProtoThread(threading.Thread):
                 print(time.strftime("%H:%M:%S"), "select OSError:", error)
                 self._want_abort = 1
 
-                log.add_warning(_("Major Socket Error: Networking terminated! %s", str(error)))
+                log.add_warning("Major Socket Error: Networking terminated! %s", str(error))
 
             except ValueError as error:
                 # Possibly opened too many sockets
@@ -1282,13 +1300,14 @@ class SlskProtoThread(threading.Thread):
 
                 if (curtime - conn_obj.lastactive) > self.IN_PROGRESS_STALE_AFTER:
 
-                    self._ui_callback([ConnectError(msg_obj)])
+                    self._ui_callback([ConnectError(msg_obj, "Timed out")])
                     self.close_connection(connsinprogress, connection_in_progress)
                     continue
 
                 try:
                     if connection_in_progress in input_list:
-                        connection_in_progress.recv(0)
+                        # Check if the socket has any data for us
+                        connection_in_progress.recv(1, socket.MSG_PEEK)
 
                 except socket.error as err:
 
@@ -1377,7 +1396,7 @@ class SlskProtoThread(threading.Thread):
                                 self._ui_callback(msgs)
 
                             if conn_obj.init is not None and conn_obj.init.type == 'F':
-                                msgs, conn_obj = self.process_file_input(conn_obj, conn_obj.ibuf)
+                                msgs, conn_obj = self.process_file_input(conn_obj, conn_obj.ibuf, conns)
                                 self._ui_callback(msgs)
 
                             if conn_obj.init is not None and conn_obj.init.type == 'D':
@@ -1392,12 +1411,32 @@ class SlskProtoThread(threading.Thread):
             # Don't exhaust the CPU
             time.sleep(0.2)
 
-        # Close Server Port
-        if server_socket is not None:
-            server_socket.close()
-
         # Networking thread aborted
 
+    def server_connect(self):
+        """ We've connected to the server """
+        self._server_disconnect = False
+
+    def server_disconnect(self):
+        """ We've disconnected from the server, clean up """
+
+        self._server_disconnect = True
+
+        for i in self._conns.copy():
+            i.close()
+            del self._conns[i]
+
+        for i in self._connsinprogress.copy():
+            i.close()
+            del self._connsinprogress[i]
+
+        while not self._queue.empty():
+            self._queue.get(0)
+
+        if not self._want_abort:
+            self._ui_callback([SetCurrentConnectionCount(0)])
+
     def abort(self):
         """ Call this to abort the thread """
         self._want_abort = True
+        self.server_disconnect()
