@@ -282,20 +282,37 @@ class Transfers:
         self.queue.put(slskmessages.SetUploadLimit(uselimit, uploadlimit, limitby))
         self.queue.put(slskmessages.SetDownloadLimit(self.eventprocessor.config.sections["transfers"]["downloadlimit"]))
 
+    def user_logged_out(self, user):
+        """ Check if a user who previously queued a file has logged out since """
+
+        try:
+            return (self.users[user].status <= 0)
+
+        except (KeyError, TypeError):
+            return False
+
     def get_user_status(self, msg):
         """ We get a status of a user and if he's online, we request a file from him """
 
         for i in self.downloads:
-            if msg.user == i.user and i.status in ["Queued", "Getting status", "User logged off", "Connection closed by peer", "Aborted", "Cannot connect", "Paused"]:
-                if msg.status != 0:
-                    if i.status not in ["Queued", "Aborted", "Cannot connect", "Paused"]:
+            if msg.user == i.user and i.status in ("Queued", "Getting status", "Establishing connection", "User logged off", "Connection closed by peer", "Aborted", "Cannot connect", "Paused"):
+                if msg.status > 0:
+                    if i.status not in ("Queued", "Aborted", "Cannot connect", "Paused"):
                         self.get_file(i.user, i.filename, i.path, i)
                 else:
-                    if i.status not in ["Aborted", "Filtered"]:
+                    if i.status not in ("Aborted", "Filtered"):
                         i.status = "User logged off"
+                        self.abort_transfer(i, send_fail_message=False)
                         self.downloadsview.update(i)
 
-        if msg.status == 0:
+        for i in self.uploads:
+            if msg.user == i.user and i.status in ("Getting status", "Establishing connection", "Initializing transfer", "Connection closed by peer", "Cannot connect"):
+                if msg.status <= 0:
+                    i.status = "User logged off"
+                    self.abort_transfer(i, send_fail_message=False)
+                    self.uploadsview.update(i)
+
+        if msg.status <= 0:
             self.check_upload_queue()
 
     def get_file(self, user, filename, path="", transfer=None, size=None, bitrate=None, length=None, checkduplicate=False):
@@ -392,6 +409,22 @@ class Transfers:
             else:
                 self.uploadsview.update(transfer)
 
+    def close_file(self, file, transfer):
+
+        transfer.file = None
+
+        if file is None:
+            return
+
+        try:
+            file.close()
+
+        except Exception as e:
+            log.add_transfer("Failed to close file %(filename)s: %(error)s", {
+                "filename": file.name,
+                "error": e
+            })
+
     def upload_failed(self, msg):
 
         for i in self.peerconns:
@@ -746,13 +779,28 @@ class Transfers:
 
     def _append_upload(self, user, filename, transferobj):
 
+        previously_queued = False
+        old_index = 0
+
         for i in self.uploads:
             if i.user == user and i.filename == filename:
+                if i.status == "Queued":
+                    # This upload was queued previously
+                    # Use the previous queue position and time
+                    transferobj.place = i.place
+                    transferobj.timequeued = i.timequeued
+                    previously_queued = True
+
                 self.uploads.remove(i)
                 self.uploadsview.remove_specific(i, True)
                 break
 
-        self.uploads.append(transferobj)
+            old_index += 1
+
+        if previously_queued:
+            self.uploads.insert(old_index, transferobj)
+        else:
+            self.uploads.append(transferobj)
 
     def file_is_upload_queued(self, user, filename):
 
@@ -1273,6 +1321,7 @@ class Transfers:
                 m = md5()
                 m.update((i.filename + i.user).encode('utf-8'))
 
+                f = None
                 pynewfname = os.path.join(incompletedir, "INCOMPLETE" + m.hexdigest() + basename)
                 try:
                     if os.access(winfname, os.F_OK):
@@ -1284,17 +1333,6 @@ class Transfers:
 
                     f = open(fname, 'ab+')
 
-                except IOError as strerror:
-                    log.add(_("Download I/O error: %s"), strerror)
-                    i.status = "Local file error"
-                    try:
-                        f.close()
-                    except Exception:
-                        pass
-                    i.conn = None
-                    self.queue.put(slskmessages.ConnClose(msg.conn))
-
-                else:
                     if self.eventprocessor.config.sections["transfers"]["lock"]:
                         try:
                             import fcntl
@@ -1308,6 +1346,16 @@ class Transfers:
                     f.seek(0, 2)
                     size = f.tell()
 
+                except IOError as strerror:
+                    log.add(_("Download I/O error: %s"), strerror)
+
+                    i.status = "Local file error"
+                    self.close_file(f, i)
+
+                    i.conn = None
+                    self.queue.put(slskmessages.ConnClose(msg.conn))
+
+                else:
                     i.currentbytes = size
                     i.file = f
                     i.place = 0
@@ -1352,6 +1400,7 @@ class Transfers:
         if i.conn is None:
             i.conn = msg.conn
             i.req = None
+            f = None
 
             if i.transfertimer is not None:
                 i.transfertimer.cancel()
@@ -1359,6 +1408,17 @@ class Transfers:
             try:
                 # Open File
                 f = open(i.realfilename, "rb")
+
+            except IOError as strerror:
+                log.add(_("Upload I/O error: %s"), strerror)
+
+                i.status = "Local file error"
+                self.close_file(f, i)
+
+                i.conn = None
+                self.queue.put(slskmessages.ConnClose(msg.conn))
+
+            else:
                 self.queue.put(slskmessages.UploadFile(i.conn, file=f, size=i.size))
                 i.status = "Initializing transfer"
                 i.file = f
@@ -1367,22 +1427,17 @@ class Transfers:
 
                 ip_address = None
                 if i.conn is not None:
-                    ip_address = i.conn.getpeername()
+                    try:
+                        ip_address = i.conn.getpeername()
+                    except OSError:
+                        # Connection already closed
+                        pass
 
                 self.log_transfer(_("Upload started: user %(user)s, IP address %(ip)s, file %(file)s") % {
                     'user': i.user,
                     'ip': ip_address,
                     'file': i.filename
                 })
-            except IOError as strerror:
-                log.add(_("Upload I/O error: %s"), strerror)
-                i.status = "Local file error"
-                try:
-                    f.close()
-                except Exception:
-                    pass
-                i.conn = None
-                self.queue.put(slskmessages.ConnClose(msg.conn))
 
             self.uploadsview.new_transfer_notification()
             self.uploadsview.update(i)
@@ -1459,11 +1514,10 @@ class Transfers:
                     needupdate = False
             except IOError as strerror:
                 log.add(_("Download I/O error: %s"), strerror)
+
                 i.status = "Local file error"
-                try:
-                    msg.file.close()
-                except Exception:
-                    pass
+                self.close_file(msg.file, i)
+
                 i.conn = None
                 self.queue.put(slskmessages.ConnClose(msg.conn))
 
@@ -1474,8 +1528,7 @@ class Transfers:
 
     def download_finished(self, file, i):
 
-        file.close()
-        i.file = None
+        self.close_file(file, i)
 
         basename = clean_file(i.filename.replace('/', '\\').split('\\')[-1])
         config = self.eventprocessor.config.sections
@@ -1522,7 +1575,6 @@ class Transfers:
             show_ui=1
         )
 
-        self.queue.put(slskmessages.ConnClose(i.conn))
         i.conn = None
 
         self.add_to_shared(newname)
@@ -1656,12 +1708,15 @@ class Transfers:
             self.eventprocessor.speed = speedbytes
             self.queue.put(slskmessages.SendUploadSpeed(speedbytes))
 
-        if file is not None:
-            file.close()
+        self.close_file(file, i)
 
         ip_address = None
         if i.conn is not None:
-            ip_address = i.conn.getpeername()
+            try:
+                ip_address = i.conn.getpeername()
+            except OSError:
+                # Connection already closed
+                pass
 
         i.status = "Finished"
         i.currentbytes = i.size
@@ -1798,17 +1853,29 @@ class Transfers:
                     mintimequeued = i.timequeued
 
         if transfercandidate is not None:
+            user = transfercandidate.user
+
             log.add_transfer(
                 "Checked upload queue, attempting to upload file %(file)s to user %(user)s", {
                     'file': transfercandidate.filename,
-                    'user': transfercandidate.user
+                    'user': user
                 }
             )
 
-            self.push_file(
-                user=transfercandidate.user, filename=transfercandidate.filename,
-                realfilename=transfercandidate.realfilename, transfer=transfercandidate
-            )
+            if self.user_logged_out(user):
+                transfercandidate.status = "User logged off"
+                self.abort_transfer(transfercandidate, send_fail_message=False)
+                self.uploadsview.update(transfercandidate)
+                self.auto_clear_upload(transfercandidate)
+
+                self.check_upload_queue()
+
+            else:
+                self.push_file(
+                    user=transfercandidate.user, filename=transfercandidate.filename,
+                    realfilename=transfercandidate.realfilename, transfer=transfercandidate
+                )
+
             self.remove_queued(transfercandidate.user, transfercandidate.filename)
 
     def place_in_queue_request(self, msg):
@@ -2007,31 +2074,32 @@ class Transfers:
             self._conn_close(conn, addr, i, "upload")
 
     def _conn_close(self, conn, addr, i, type):
+
         if i.requestconn == conn and i.status == "Requesting file":
+            # This code is probably not needed anymore?
             i.requestconn = None
             i.status = "Connection closed by peer"
             i.req = None
 
-        if i.file is not None:
-            i.file.close()
-
-        i.conn = None
-        i.legacy_attempt = False
+        self.abort_transfer(i, send_fail_message=False)  # Don't send "Aborted" message, let remote user recover
 
         if i.status != "Finished":
-            if i.user in self.users and self.users[i.user].status == 0:
-                i.status = "User logged off"
+            if type == "download":
+                if self.user_logged_out(i.user):
+                    i.status = "User logged off"
+                else:
+                    i.status = "Connection closed by peer"
 
-            elif type == "download":
-                i.status = "Connection closed by peer"
-
-            elif type == "upload" and i.status == "Transferring":
+            elif type == "upload" and i.status != "Queued":
                 """ Only cancel files being transferred, queued files will take care of
                 themselves. We don't want to cancel all queued files at once, in case
                 it's just a connectivity fluke. """
 
-                i.status = "Cancelled"
-                self.abort_transfer(i, send_fail_message=False)  # Don't send "Aborted" message, let remote user recover
+                if self.user_logged_out(i.user):
+                    i.status = "User logged off"
+                else:
+                    i.status = "Cancelled"
+
                 self.auto_clear_upload(i)
 
         curtime = time.time()
@@ -2090,14 +2158,9 @@ class Transfers:
                 continue
 
             i.status = "Local file error"
-
-            try:
-                msg.file.close()
-            except Exception:
-                pass
+            self.close_file(msg.file, i)
 
             i.conn = None
-            self.queue.put(slskmessages.ConnClose(msg.conn.conn))
             log.add(_("I/O error: %s"), msg.strerror)
 
             if i in self.downloads:
@@ -2187,6 +2250,39 @@ class Transfers:
 
         return destination
 
+    def retry_download(self, transfer):
+
+        if transfer.status in ("Finished", "Old"):
+            return
+
+        user = transfer.user
+
+        if self.user_logged_out(user):
+            transfer.status = "User logged off"
+            self.abort_transfer(transfer, send_fail_message=False)
+            self.downloadsview.update(transfer)
+            return
+
+        self.abort_transfer(transfer)
+        self.get_file(user, transfer.filename, transfer.path, transfer)
+
+    def retry_upload(self, transfer):
+
+        user = transfer.user
+
+        if user in self.get_transferring_users():
+            return
+
+        if self.user_logged_out(user):
+            transfer.status = "User logged off"
+            self.abort_transfer(transfer, send_fail_message=False)
+            self.uploadsview.update(transfer)
+            self.auto_clear_upload(transfer)
+            return
+
+        self.eventprocessor.send_message_to_peer(user, slskmessages.UploadQueueNotification(None))
+        self.push_file(user, transfer.filename, transfer.path, transfer=transfer)
+
     def abort_transfers(self, send_fail_message=True):
         """ Stop all transfers """
 
@@ -2198,7 +2294,7 @@ class Transfers:
                 self.abort_transfer(i, send_fail_message=send_fail_message)
                 i.status = "Old"
 
-    def abort_transfer(self, transfer, remove=False, reason="Aborted", send_fail_message=True):
+    def abort_transfer(self, transfer, reason="Aborted", send_fail_message=True):
 
         transfer.legacy_attempt = False
         transfer.req = None
@@ -2216,14 +2312,7 @@ class Transfers:
             transfer.transfertimer.cancel()
 
         if transfer.file is not None:
-            try:
-                transfer.file.close()
-                if remove:
-                    os.remove(transfer.file.name)
-            except Exception:
-                pass
-
-            transfer.file = None
+            self.close_file(transfer.file, transfer)
 
             if transfer in self.uploads:
                 self.log_transfer(
